package de.msponer.android.yac.activities;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.Surface;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;

import com.google.zxing.integration.android.IntentIntegrator;
import com.google.zxing.integration.android.IntentResult;

import de.msponer.android.yac.R;
import de.msponer.android.yac.activities.position.WorkQueueMapActivity;
import de.msponer.android.yac.activities.workqueues.LocalizedWorkListActivity;
import de.msponer.android.yac.activities.workqueues.SearchResultListActivity;
import de.msponer.android.yac.activities.workqueues.WorkQueuesTabHostActivity;

/**
 * Home-Activity. Is instantiated and displayed on application start. The most
 * important functionalities are accessible. Chosen layout is a dashboard.
 * 
 * @author msponer
 */
public class HomeActivity extends Activity {

	/**
	 * Menu-Id for entry 'Preferences'.
	 */
	public static final int MENU_PREFERENCES = Menu.FIRST;

	/**
	 * Activity tag.
	 */
	private static final String TAG = "HomeActivity";

	/**
	 * {@inheritDoc} Called when the activity is created.
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentViewBasedOnOrientation();

		initButtons();

		// TODO: Feature: Autosync on startup
		// New feature, will be implemented soon ...
		// SharedPreferences sharedPreferences =
		// getSharedPreferences("de.msponer.android.yac_preferences",
		// MODE_PRIVATE);
		//
		// boolean autosync = sharedPreferences.getBoolean("prefs_autosync",
		// false);
	}

	/**
	 * Checks the device orientation and chooses either a vertical or a
	 * horizontal layout.
	 */
	private void setContentViewBasedOnOrientation() {
		WindowManager winMan = (WindowManager) getSystemService(Context.WINDOW_SERVICE);

		if (winMan != null) {
			int orientation = winMan.getDefaultDisplay().getOrientation();

			if (orientation == Surface.ROTATION_0
					|| orientation == Surface.ROTATION_180) {
				// Portrait
				setContentView(R.layout.home_vert);
			} else if (orientation == Surface.ROTATION_90
					|| orientation == Surface.ROTATION_270) {
				// Landscape
				setContentView(R.layout.home_hori);
			}
		}
	}

	/**
	 * Initializes all activity buttons.
	 */
	private void initButtons() {

		Button syncButton = (Button) findViewById(R.id.buttonSync);
		syncButton.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				Intent prefsIntent = new Intent(v.getContext(),
						SyncActivity.class);
				startActivity(prefsIntent);
			}
		});

		Button workQueuesButton = (Button) findViewById(R.id.buttonWorkQueues);
		workQueuesButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				Intent workQueuesIntent = new Intent(v.getContext(),
						WorkQueuesTabHostActivity.class);
				startActivity(workQueuesIntent);
			}
		});

		Button preferencesButton = (Button) findViewById(R.id.buttonPreferences);
		preferencesButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				Intent prefsIntent = new Intent(v.getContext(),
						PreferencesActivity.class);
				startActivity(prefsIntent);
			}
		});

		Button qrScanButton = (Button) findViewById(R.id.buttonQRScan);
		qrScanButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				// Intent intent = new
				// Intent("com.google.zxing.client.android.SCAN");
				// intent.setPackage("com.google.zxing.client.android");
				// intent.putExtra("SCAN_MODE", "QR_CODE_MODE");
				// startActivityForResult(intent, 0);
				IntentIntegrator.initiateScan(HomeActivity.this);
			}
		});

		Button locationButton = (Button) findViewById(R.id.buttonLocation);
		locationButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				Intent locIntent = new Intent(v.getContext(),
						LocalizedWorkListActivity.class);
				startActivity(locIntent);

			}
		});

		Button workQueueMapButton = (Button) findViewById(R.id.buttonWorkQueuemap);
		workQueueMapButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				Intent workQueueMapIntent = new Intent(v.getContext(),
						WorkQueueMapActivity.class);
				startActivity(workQueueMapIntent);

			}
		});

		ImageButton searchButton = (ImageButton) findViewById(R.id.imageButtonSearchWork);
		searchButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				EditText searchPattern = (EditText) findViewById(R.id.editTextSearchPattern);
				String searchPatternString = searchPattern.getText().toString();

				if (searchPatternString != null
						&& searchPatternString.length() > 0) {
					searchWorkitems(searchPatternString);
				}
			}
		});

		syncButton.requestFocus();
	}

	/**
	 * {@inheritDoc} Invokes action on activity result. QR-Code IntentIntegrator
	 * request code is handled -> Work-Item search is invoked.
	 */
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		switch (requestCode) {
		
		case IntentIntegrator.REQUEST_CODE: 
			if (resultCode != RESULT_CANCELED) {
				IntentResult scanResult = IntentIntegrator.parseActivityResult(
						requestCode, resultCode, data);
				if (scanResult != null) {
					String upc = scanResult.getContents();
					Log.i(TAG, scanResult.getFormatName());
					Log.i(TAG, upc);

					searchWorkitems(upc);
				}
			}
			break;
			
		default: 
			break;
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.menu_home, menu);
		return super.onCreateOptionsMenu(menu);
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.menu_preferences:
			Intent preferencesIntent = new Intent(this,
					PreferencesActivity.class);
			startActivity(preferencesIntent);
		default:
		}

		return super.onOptionsItemSelected(item);
	}

	/**
	 * Prepares the scan result content string and invokes SearchResultListActivity.
	 * @param scanResultContent The QR-Code scan result content string
	 */
	private void searchWorkitems(String scanResultContent) {

		Log.d(TAG, "Seraching for pattern: " + scanResultContent);

		if (!scanResultContent.startsWith("%")) {
			scanResultContent = "%" + scanResultContent;
		}
		if (!scanResultContent.endsWith("%")) {
			scanResultContent = scanResultContent + "%";
		}

		Intent scanResultsIntent = new Intent(this,
				SearchResultListActivity.class);
		scanResultsIntent.putExtra("SEARCH_PATTERN", scanResultContent);
		startActivity(scanResultsIntent);

	}

}