package de.msponer.android.yac.activities.images;

import java.io.ByteArrayOutputStream;

import android.graphics.Bitmap;
import android.graphics.Bitmap.CompressFormat;
import android.graphics.Matrix;
import android.util.Base64;
import android.util.Log;

/**
 * Class for resizing bitmaps.
 * 
 * @author msponer
 */
public class BitmapResizer {

	/**
	 * The resize factor is 0.9. Images are resized in 10 percent steps until
	 * they fit the screen.
	 */
	public static final double RESIZE_FACTOR = 0.9;

	/**
	 * The JPEG compression level: 50.
	 */
	public static final int COMPRESSION_LEVEL = 50;

	/**
	 * Resizes a given bitmap that it fits the specified width and height and
	 * returns the resized image encoded with BASE64.
	 * 
	 * @param mBitmap
	 *            The bitmap to resize.
	 * @param displayWidth
	 *            The display width in pixels.
	 * @param displayHeight
	 *            The display height in pixels.
	 * @return The resized bitmap encoded with BASE64
	 */
	public String getBase64EncodedResizedBitmap(Bitmap mBitmap, int displayWidth, 
			int displayHeight) {

		int width = mBitmap.getWidth();
		int height = mBitmap.getHeight();

		Log.d("BitmapResizer", "w,h: " + width + "," + height);

		int newWidth = width;
		int newHeight = height;

		while (newWidth >= displayWidth || newHeight >= displayHeight) {
			newWidth *= RESIZE_FACTOR;
			newHeight *= RESIZE_FACTOR;

			Log.d("BitmapResizer", "w,h: " + newWidth + "," + newHeight);
		}

		// calculate the scale - in this case
		float scaleWidth = ((float) newWidth) / width;
		float scaleHeight = ((float) newHeight) / height;

		// createa matrix for the manipulation
		Matrix matrix = new Matrix();
		// resize the bit map
		matrix.postScale(scaleWidth, scaleHeight);

		// recreate the new Bitmap
		Bitmap resizedBitmap = Bitmap.createBitmap(mBitmap, 0, 0, width, height, matrix, true);

		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		resizedBitmap.compress(CompressFormat.JPEG, COMPRESSION_LEVEL, bos);
		byte[] bitmapdata = bos.toByteArray();

		String base64Binary = Base64.encodeToString(bitmapdata, 0);

		return base64Binary;
	}

}
