/**
 * 
 */
package de.msponer.android.yac.activities.images;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.Display;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.Toast;
import de.msponer.android.yac.R;

/**
 * Activity for creating touch screen signatures.
 * 
 * @author msponer
 */
public class SigningActivity extends Activity {

	/**
	 * Menu ID for the command to save as bitmap.
	 */
	private static final int SAVE_BITMAP_ID = Menu.FIRST;

	/**
	 * The view component for painting the signature.
	 */
	private SigningView mSigningView;
	
	/**
	 * The button to save the signature.
	 */
	private Button mSaveButton;

	/**
	 * The view id.
	 */
	private int mViewId;

	/**
	 * The handler to invalidate the view.
	 */
	private Handler mHandler = new Handler() {
		public void handleMessage(android.os.Message msg) {
			mSigningView.invalidate();
		};
	};

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		Bundle extras = getIntent().getExtras();
		mViewId = extras.getInt("VIEW_ID");

		setContentView(R.layout.signing);

		mSigningView = (SigningView) findViewById(R.id.signingView);

		this.mSaveButton = (Button) findViewById(R.id.buttonSaveSign);
		this.mSaveButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				saveBitmap();
			}
		});

		setHandler(new Handler());
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		menu.add(0, SAVE_BITMAP_ID, 0, "Save");
		return super.onCreateOptionsMenu(menu);
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case SAVE_BITMAP_ID:
			saveBitmap();
			return true;
		default:
			return super.onOptionsItemSelected(item);
		}
	}

	/**
	 * Saves the painting as bitmap and returns the resulting BASE64 string via intent.
	 * Activity is finished after save process.
	 */
	private void saveBitmap() {

		try {

			Bitmap mBitmap = mSigningView.getBitmap();

			Display display = getWindowManager().getDefaultDisplay();
			int width = display.getWidth();
			int height = display.getHeight();

			Log.d("SIGNING", "display-dimensions: " + width + "," + height);

			String base64Binary = new BitmapResizer().getBase64EncodedResizedBitmap(mBitmap, width,
					height);

			Log.d("SIGNING", base64Binary);

			Intent mIntent = new Intent();

			mIntent.putExtra("VIEW_ID", mViewId);
			mIntent.putExtra("BASE64_BINARY", base64Binary);
			setResult(RESULT_OK, mIntent);

			finish();

		} catch (Throwable ex) {
			Toast.makeText(getApplicationContext(), "Error: " + ex.getMessage(), Toast.LENGTH_LONG)
					.show();
			ex.printStackTrace();
		}
	}
	
	/**
	 * Return the current activity handler.
	 * @return The current handler.
	 */
	public Handler getHandler() {
		return mHandler;
	}
	
	/**
	 * Sets the current activity handler.
	 * @param handler The handler to set.
	 */
	public void setHandler(Handler handler) {
		this.mHandler = handler;
	}

}
