/**
 * 
 */
package de.msponer.android.yac.activities.position;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.OverlayItem;

import de.msponer.android.yac.model.MobileWorkItem;

/**
 * Google maps overlay for mobile work items.
 * 
 * @author msponer
 * 
 */
public class MobileWorkItemOverlay extends OverlayItem {

	/**
	 * The mobile work item.
	 */
	private MobileWorkItem mobileWorkItem;

	/**
	 * Constructor using geo point, title and snippet.
	 * 
	 * @param point
	 *            The geo point of the mobile work item.
	 * @param title
	 *            The title of the mobile work item.
	 * @param snippet
	 *            The text snippet of the mobile work item.
	 */
	public MobileWorkItemOverlay(GeoPoint point, String title, String snippet) {
		super(point, title, snippet);
	}

	/**
	 * Constructor setting geo point and mobile work item.
	 * 
	 * @param point
	 *            The geo point of the mobile work item.
	 * @param mobWorkItem
	 *            The mobile work item for this overlay.
	 */
	public MobileWorkItemOverlay(GeoPoint point, MobileWorkItem mobWorkItem) {
		super(point, mobWorkItem.getID(), mobWorkItem.getTaskName());
		this.mobileWorkItem = mobWorkItem;
	}

	/**
	 * Sets the current mobile work item.
	 * 
	 * @param mobWorkItem
	 *            The current mobile work item.
	 */
	public void setMobileWorkItem(MobileWorkItem mobWorkItem) {
		this.mobileWorkItem = mobWorkItem;
	}

	/**
	 * Return the current mobile work item.
	 * 
	 * @return The current mobile work item.
	 */
	public MobileWorkItem getMobileWorkItem() {
		return mobileWorkItem;
	}
}
