/**
 * 
 */
package de.msponer.android.yac.activities.position;

import java.util.ArrayList;
import java.util.List;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Canvas;
import android.graphics.drawable.Drawable;

import com.google.android.maps.ItemizedOverlay;
import com.google.android.maps.MapView;

import de.msponer.android.yac.R;
import de.msponer.android.yac.activities.workqueues.WorkItemActivity;
import de.msponer.android.yac.model.MobileWorkItem;

/**
 * List of Google maps overlays locating mobile work items. Implementing
 * tappable functionality.
 * 
 * @param <Item> The superclass that is extended.
 * @author msponer
 */
public class TappableWorkListOverlay<Item extends MobileWorkItemOverlay> extends
		ItemizedOverlay<MobileWorkItemOverlay> {

	/**
	 * The list of mobile work items to display.
	 */
	private List<MobileWorkItemOverlay> mWorkItemOverlays = new ArrayList<MobileWorkItemOverlay>();

	/**
	 * The activity context.
	 */
	private Context mContext;

	/**
	 * Constructor setting the default marker.
	 * 
	 * @param defaultMarker
	 *            The default marker.
	 */
	public TappableWorkListOverlay(Drawable defaultMarker) {
		super(boundCenterBottom(defaultMarker));
	}

	/**
	 * Constructor setting the default marker and the activity context.
	 * 
	 * @param defaultMarker
	 *            The default marker.
	 * @param context
	 *            The activity context.
	 */
	public TappableWorkListOverlay(Drawable defaultMarker, Context context) {
		super(boundCenterBottom(defaultMarker));
		this.mContext = context;
	}

	@Override
	protected MobileWorkItemOverlay createItem(int i) {
		return mWorkItemOverlays.get(i);
	}

	@Override
	public int size() {
		return mWorkItemOverlays.size();
	}

	/**
	 * Adds a new overlay element to the list and populates it.
	 * 
	 * @param overlay The overlay to add.
	 */
	public void addOverlay(Item overlay) {
		mWorkItemOverlays.add(overlay);
		populate();
	}

	/**
	 * {@inheritDoc} Implements the tap functionality. Opens an AlertDialog on
	 * tapping an overlay object and allows the user to start the editing
	 * process.
	 * 
	 * @param index
	 *            The tapped index.
	 * @return Returns true.
	 */
	@Override
	protected boolean onTap(int index) {
		final MobileWorkItemOverlay mobileWorkItemOverlay = this.mWorkItemOverlays.get(index);

		AlertDialog.Builder builder = new AlertDialog.Builder(this.mContext);

		builder.setTitle(mobileWorkItemOverlay.getTitle())
				.setMessage(mobileWorkItemOverlay.getSnippet() + "\n" + "Start editing?")
				.setCancelable(false).setIcon(R.drawable.androidmarker)
				.setPositiveButton("Yes", new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int id) {
						startWorkItemActivity(mobileWorkItemOverlay.getMobileWorkItem());
					}
				}).setNegativeButton("No", new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int id) {
						dialog.cancel();
					}
				});
		AlertDialog dialog = builder.create();
		dialog.show();
		return true;
	}

	/**
	 * Starts the work item activity.
	 * 
	 * @param mobileWorkItem
	 *            The mobile work item that is passed to the activity.
	 */
	protected void startWorkItemActivity(MobileWorkItem mobileWorkItem) {
		Intent workItemIntent = new Intent(this.mContext, WorkItemActivity.class);

		workItemIntent.putExtra("PRIMARY_KEY", mobileWorkItem.getPrimaryKey());
		workItemIntent.setAction(WorkItemActivity.EDIT_INTENT);

		this.mContext.startActivity(workItemIntent);
	}

	@Override
	public void draw(Canvas canvas, MapView mapView, boolean shadow) {
		super.draw(canvas, mapView, false);
	}

}
