/**
 * 
 */
package de.msponer.android.yac.activities;

import java.sql.SQLException;
import java.util.List;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.ComponentName;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.j256.ormlite.android.apptools.OrmLiteBaseActivity;
import com.j256.ormlite.dao.Dao;

import de.msponer.android.yac.R;
import de.msponer.android.yac.adapter.WorkItemSyncAdapter;
import de.msponer.android.yac.db.DatabaseHelper;
import de.msponer.android.yac.model.MobileWorkItem;
import de.msponer.android.yac.model.State;
import de.msponer.android.yac.services.impl.YConnectionService;
import de.msponer.android.yac.services.impl.YConnectionService.YConnectionServiceBinder;

/**
 * Activity to synchronize YAC with a remote YAWL Resource-Service. Responsible
 * for the communication between YAC and YAWL.
 * 
 * Uses YAWL WorkQueue- and Resource-Gateway.
 * 
 * Two Sync-modes are supported: Complete-Sync and Live-Sync
 * 
 * @author msponer
 */
public class SyncActivity extends OrmLiteBaseActivity<DatabaseHelper> {

	/**
	 * Activity tag.
	 */
	private static final String TAG = "SyncActivity";

	/**
	 * Auto-Sync tag.
	 */
	public static final String AUTOSYNC = "AUTOSYNC";

	/**
	 * The Sync-Button.
	 */
	private Button mSyncButton;

	/**
	 * The Reset-DB button.
	 */
	private Button mResetDbButton;

	/**
	 * The TextView for displaying sync results.
	 */
	private TextView mSyncResults;

	/**
	 * The MobileWorkItem DAO.
	 */
	private Dao<MobileWorkItem, Integer> mobileWorkItemDao;

	/**
	 * The WorkItemSyncAdapter.
	 */
	private WorkItemSyncAdapter mWorkItemSyncAdapter;

	/**
	 * The Work-Item sync list.
	 */
	private List<MobileWorkItem> mWorkItemSyncList;

	/**
	 * The connection intent.
	 */
	private Intent mYConnectionIntent;

	/**
	 * The YConnectionService.
	 */
	private YConnectionService mYConnectionService;

	/**
	 * The ProgressDialog.
	 */
	private ProgressDialog mDialog;

	/**
	 * The ListView for items to sync.
	 */
	private ListView mListViewSyncItems;

	/**
	 * The sync result handler.
	 */
	private final Handler mSyncResultHandler = new Handler() {
		public void handleMessage(Message msg) {
			final Bundle bundle = msg.getData();
			final boolean success = bundle.getBoolean("SYNC_RESULT");

			if (success) {
				((TextView) findViewById(R.id.textViewSyncResult))
						.setText("Sync successfully complete!");
			} else {
				((TextView) findViewById(R.id.textViewSyncResult)).setText("Sync incomplete!");
			}

			String syncOutput = bundle.getString("SYNC_OUTPUT");
			((TextView) findViewById(R.id.textViewSyncOutput)).setText(syncOutput);

			resetSyncItems();

			if (mDialog.isShowing()) {
				mDialog.dismiss();
			}
			if (mYConnectionService.isRgConnected() || mYConnectionService.isWqgConnected()) {
				mYConnectionService.yDisconnect();
			}
		};
	};

	/**
	 * The service connection.
	 */
	private ServiceConnection mYConnection = new ServiceConnection() {

		@Override
		public void onServiceConnected(ComponentName name, IBinder service) {
			YConnectionServiceBinder binder = (YConnectionServiceBinder) service;
			binder.setActivityCallbackHandler(mSyncResultHandler);
			mYConnectionService = ((YConnectionServiceBinder) service).getYConnectionService();

			Toast.makeText(getApplicationContext(), "YConnectionService connected.",
					Toast.LENGTH_SHORT).show();

			// Intent callingIntent = getIntent();
			// String intentAction = callingIntent.getAction();
			//
			// if(intentAction.equals(SyncActivity.AUTOSYNC)){
			// syncDB();
			// finish();
			// }
		}

		@Override
		public void onServiceDisconnected(ComponentName name) {
			mYConnectionService = null;
			Toast.makeText(getApplicationContext(), "YConnectionService disconnected.",
					Toast.LENGTH_SHORT).show();
		}
	};

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);

		setContentView(R.layout.sync);

		mSyncResults = ((TextView) findViewById(R.id.textViewSyncResult));
		mSyncResults.setText("");

		mYConnectionIntent = new Intent(this, YConnectionService.class);

		this.mSyncButton = (Button) findViewById(R.id.buttonSync);
		this.mSyncButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {

				mSyncResults = (TextView) findViewById(R.id.textViewSyncResult);
				mSyncResults.setText("");

				syncDB();
			}
		});

		this.mResetDbButton = (Button) findViewById(R.id.buttonResetDB);
		this.mResetDbButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				resetDb();

			}
		});

		mListViewSyncItems = (ListView) findViewById(R.id.listViewSyncItems);
		// TextView emptyMessage = new TextView(this);
		// emptyMessage.setText("No workitems found to sync");
		// listViewSyncItems.setEmptyView(emptyMessage);
		mListViewSyncItems.setEmptyView(findViewById(R.id.textViewNoSyncItems));

		createSyncList();

	}

	@Override
	protected void onResume() {
		bindService(mYConnectionIntent, mYConnection, BIND_AUTO_CREATE);
		Log.d(TAG, "Binding Service: " + YConnectionService.class.getName());
		super.onResume();
	}

	@Override
	protected void onPause() {
		unbindService(mYConnection);
		Log.d(TAG, "Unbinding Service: " + YConnectionService.class.getName());
		super.onPause();
	}

	/**
	 * Determines work items to sync and sets up the sync list.
	 */
	private void createSyncList() {

		setupSyncItems();

		this.mWorkItemSyncAdapter = new WorkItemSyncAdapter(this, R.layout.list_item_queues,
				this.mWorkItemSyncList);

		mListViewSyncItems.setTextFilterEnabled(true);

		if (this.mWorkItemSyncAdapter != null) {
			if (mListViewSyncItems != null) {
				mListViewSyncItems.setAdapter(this.mWorkItemSyncAdapter);
			} else {
				Log.d(TAG, "listviewsyncitems == null");
			}

		} else {
			Log.d(TAG, "workitemsyncadapter == null");
		}

		mListViewSyncItems.setOnItemClickListener(new OnItemClickListener() {

			@Override
			public void onItemClick(AdapterView<?> arg0, View view, int position, long arg3) {

				final MobileWorkItem selectedWorkItem = mWorkItemSyncAdapter.getItem(position);

				AlertDialog.Builder builder = new AlertDialog.Builder(SyncActivity.this);
				builder.setMessage("Do you really want to delete the selected sync-workitem?")
						.setCancelable(false)
						.setPositiveButton("Yes", new DialogInterface.OnClickListener() {
							public void onClick(DialogInterface dialog, int id) {

								deleteSyncItem(selectedWorkItem);
							}
						}).setNegativeButton("No", new DialogInterface.OnClickListener() {
							public void onClick(DialogInterface dialog, int id) {
								dialog.cancel();
							}
						});
				AlertDialog alert = builder.create();
				alert.show();
			}
		});
	}

	/**
	 * Deletes a specified mobile work item form database and removes the entry from the sync list. 
	 * @param item The mobile work item to delete.
	 */
	private void deleteSyncItem(MobileWorkItem item) {

		try {

			this.mWorkItemSyncList.remove(item);
			this.mobileWorkItemDao.delete(item);
			this.mWorkItemSyncAdapter.notifyDataSetChanged();

			Toast.makeText(getApplicationContext(), "Deleted sync-workitem: " + item.getID(),
					Toast.LENGTH_SHORT).show();
		} catch (SQLException e) {
			Log.e(TAG, "Error while deleting the specified work item:  " + item.getID());
		}

	}

	/**
	 * Resets the sync items and notifies sync adapter.
	 */
	private void resetSyncItems() {

		setupSyncItems();

		this.mWorkItemSyncAdapter = new WorkItemSyncAdapter(this, R.layout.list_item_queues,
				this.mWorkItemSyncList);
		this.mListViewSyncItems.setAdapter(mWorkItemSyncAdapter);
		this.mWorkItemSyncAdapter.notifyDataSetChanged();
	}

	/**
	 * Sets up items to sync. Queries database for modified work items and saves the resulting list.
	 */
	private void setupSyncItems() {
		try {
			this.mobileWorkItemDao = getHelper().getMobileWorkItemDao();

			// Query DB for WorkItems
			this.mWorkItemSyncList = mobileWorkItemDao.queryForEq("mState", State.MODIFIED);
		} catch (SQLException e) {
			Log.e(TAG, "Error during sync items setup.");
		}
	}

	/**
	 * Starts a complete synchronization between YAC and YAWL.
	 * Establishes the connections to the required YAWL gateways and invokes sync process over the
	 * service connection.
	 */
	private void syncDB() {
		setProgressBarIndeterminateVisibility(true);

		Log.d(TAG, "Establishing YAWL-Connection...");
		if (mYConnectionService != null) {
			mYConnectionService.yConnect();
		}

		if (mYConnectionService.isWqgConnected()) {
			Log.d(TAG, "WQG-Connection: Successful!");
			((TextView) findViewById(R.id.textViewCurrentRGHandle)).setText(mYConnectionService
					.getCurrentRGHandle());
		}

		if (mYConnectionService.isRgConnected()) {
			Log.d(TAG, "RG-Connection: Successful!");
			((TextView) findViewById(R.id.textViewCurrentWQGHandle)).setText(mYConnectionService
					.getCurrentWQGHandle());
		}
		setProgressBarIndeterminateVisibility(false);
		mDialog = ProgressDialog.show(SyncActivity.this, "Please wait..", "Sync in action", true,
				false);
		mYConnectionService.doSync();
	}

	/**
	 * Invokes a database reset. 
	 */
	private void resetDb() {
		mYConnectionService.resetDb();
		((TextView) findViewById(R.id.textViewSyncResult)).setText("Database reseted");

		resetSyncItems();
	}
}
