/**
 * 
 */
package de.msponer.android.yac.activities.workqueues;

import java.io.ByteArrayOutputStream;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import org.jdom.Element;
import org.yawlfoundation.yawl.util.JDOMUtil;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.Bitmap.CompressFormat;
import android.graphics.BitmapFactory;
import android.graphics.Typeface;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.text.InputType;
import android.util.Base64;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup.LayoutParams;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.DatePicker;
import android.widget.DatePicker.OnDateChangedListener;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ImageView.ScaleType;
import android.widget.LinearLayout;
import android.widget.Spinner;
import android.widget.TextView;

import com.j256.ormlite.android.apptools.OrmLiteBaseActivity;
import com.j256.ormlite.dao.Dao;

import de.msponer.android.yac.R;
import de.msponer.android.yac.activities.images.CameraActivity;
import de.msponer.android.yac.activities.images.SigningActivity;
import de.msponer.android.yac.activities.workqueues.listener.AddWorkItemElementEntryListener;
import de.msponer.android.yac.activities.workqueues.listener.RemoveWorkItemElementEntryListener;
import de.msponer.android.yac.db.DatabaseHelper;
import de.msponer.android.yac.model.MobileWorkItem;
import de.msponer.android.yac.model.State;
import de.msponer.android.yac.util.QRCodeDecoder;
import de.msponer.android.yac.xml.WorkItemElementEntry;
import de.msponer.android.yac.xml.WorkItemElementEntryDeserializer;
import de.msponer.android.yac.xml.WorkItemElementEntrySerializer;

/**
 * Activity to edit a mobile work item. Content view is generated by a given XSD
 * and XML document.
 * 
 * @author marcel
 * 
 */
public class EditWorkItemActivity extends OrmLiteBaseActivity<DatabaseHelper> {
	/**
	 * The activity tag.
	 */
	private static final String TAG = "EditWorkItemActivity";

	/**
	 * Request code for a single (explicit) QR code scan operation.
	 */
	public static final int QR_SCAN_SINGLE = 10;

	/**
	 * Request code for a complete (implicit) QR code scan operation.
	 */
	public static final int QR_SCAN_COMPLETE = 11;

	/**
	 * Request code for taking a picture.
	 */
	public static final int TAKE_PICTUTE = 12;

	/**
	 * Request code indicating a touch screen signing.
	 */
	public static final int SIGN = 13;

	/**
	 * Result code for operation: 'Complete'.
	 */
	public static final int COMPLETED = 101;

	/**
	 * Result code for operation: 'Saved'.
	 */
	public static final int SAVED = 102;

	/**
	 * The image compression level.
	 */
	private static final int COMPRESSION_LEVEL = 50;

	/**
	 * The current mobile work item id.
	 */
	private int mCurrentMobileWorkItemId;

	/**
	 * The current mobile work item.
	 */
	private MobileWorkItem mCurrentMobileWorkItem;

	/**
	 * The current work item element entry root.
	 */
	private WorkItemElementEntry mWorkItemElementEntryRoot;

	/**
	 * The unique view id.
	 */
	private int mUniqueViewId = 100;

	/**
	 * The mobile work item dao.
	 */
	private Dao<MobileWorkItem, Integer> mMobileWorkitemDao;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		this.mCurrentMobileWorkItemId = getIntent().getExtras().getInt("PRIMARY_KEY");

		try {
			mMobileWorkitemDao = getHelper().getDao(MobileWorkItem.class);

			// Queries the mobile work item
			mCurrentMobileWorkItem = mMobileWorkitemDao.queryForId(mCurrentMobileWorkItemId);

			// Choose data string
			String dataString;
			if (mCurrentMobileWorkItem.getUpdatedDataString() != null
					&& mCurrentMobileWorkItem.getUpdatedDataString().length() > 0) {
				dataString = mCurrentMobileWorkItem.getUpdatedDataString();
			} else {
				dataString = mCurrentMobileWorkItem.getDataListString();
			}

			try {
				// Deserialize XSD and XML document
				this.mWorkItemElementEntryRoot = WorkItemElementEntryDeserializer.parse(
						mCurrentMobileWorkItem.getDataSchemaString(), dataString);

			} catch (Exception e) {
				e.printStackTrace();
			}

			setTitle(mWorkItemElementEntryRoot.getName());

			setContentView(R.layout.edit_workitem);

			initContentView();

		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Initializes the content view.
	 */
	private void initContentView() {

		LinearLayout linearLayoutFields = 
			(LinearLayout) findViewById(R.id.linearLayoutEditWorkitemFields);

		createViewElements(linearLayoutFields, mWorkItemElementEntryRoot);

		Button btnSave = (Button) findViewById(R.id.buttonEditSave);
		if (btnSave != null) {
			btnSave.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					saveAndExit();
				}
			});
		}

		Button btnComplete = (Button) findViewById(R.id.buttonEditComplete);
		btnComplete.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				complete();
			}
		});

		Button btnScan = (Button) findViewById(R.id.buttonEditScan);
		btnScan.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				scanCode(QR_SCAN_COMPLETE);
			}
		});

	}

	/**
	 * Creates view elements based on the work element entries and adds them to
	 * the given father layout.
	 * 
	 * @param fatherLayout
	 *            The father layout to which items are added.
	 * @param root
	 *            The work element entry root.
	 */
	public void createViewElements(LinearLayout fatherLayout, WorkItemElementEntry root) {

		List<WorkItemElementEntry> list = root.getSubElementEntryList();

		// Iterate over all workElementEntries
		for (WorkItemElementEntry workItemElementEntry : list) {
			Log.d(TAG, "Name: " + workItemElementEntry.getName());

			final LinearLayout childLayout = new LinearLayout(this);
			childLayout.setOrientation(LinearLayout.VERTICAL);

			// Setting key
			TextView key = new TextView(this);
			String currentKey = workItemElementEntry.getName();
			key.setText(currentKey);
			key.setId(getUniqueViewId());
			childLayout.addView(key);

			// Setting value
			if (workItemElementEntry.getDatatype() != null) {
				String datatype = workItemElementEntry.getDatatype();
				String currentValue = workItemElementEntry.getValue();
				
				Log.d(TAG, "DataType: " + datatype);
				Log.d(TAG, "currentValue: " + currentValue);
				
				// Datatype: BOOLEAN
				if (datatype.equals("xsd:boolean")) {
					CheckBox checkBox = new CheckBox(this);
					checkBox.setId(getUniqueViewId());
					workItemElementEntry.setViewId(checkBox.getId());
					
					if (currentValue != null) {
						if (currentValue.equals("true")) {
							checkBox.setChecked(true);
						} else {
							checkBox.setChecked(false);
						}
					} else {
						checkBox.setChecked(false);
					}
					
					childLayout.addView(checkBox);

					// Datatype: DATE
				} else if (datatype.equals("xsd:date")) {

					DatePicker date = new DatePicker(this);
					date.setId(getUniqueViewId());
					workItemElementEntry.setViewId(date.getId());
					Log.d(TAG, "Date-Value: " + currentValue);

					// Convert the date that it fits the components needs.
					if (currentValue != null && currentValue.length() > 9) {
						String yearString = currentValue.substring(0, 4);
						String monthString = currentValue.substring(5, 7);
						String dayString = currentValue.substring(8, 10);
						Log.d(TAG, yearString + "-" + monthString + "-" + dayString);

						try {
							int year = Integer.parseInt(yearString);
							int month = Integer.parseInt(monthString);
							int day = Integer.parseInt(dayString);

							date.init(year, month - 1, day, new OnDateChangedListener() {
								@Override
								public void onDateChanged(DatePicker view, int year,
										int monthOfYear, int dayOfMonth) {
								}
							});
						} catch (Exception e) {
							Log.e(TAG, e.getMessage(), e.fillInStackTrace());
						}
					}
					childLayout.addView(date);

					// Datatype: NUMERIC
				} else if (datatype.equals("xsd:integer") || datatype.equals("xsd:long")) {

					ScanableEditText value = new ScanableEditText(this);
					value.setId(getUniqueViewId());

					workItemElementEntry.setViewId(value.getId());

					value.setInputType(InputType.TYPE_CLASS_NUMBER);

					value.setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT,
							LayoutParams.WRAP_CONTENT));

					if (currentValue != null) {
						value.setText(currentValue);
					}
					childLayout.addView(value);

					// Datatype: BASE64BINARY
				} else if (datatype.equals("xsd:base64Binary")) {

					final ImageButton imageButton = new ImageButton(this);
					imageButton.setDrawingCacheEnabled(true);

					if (currentValue != null && currentValue.length() > 0) {
						byte[] imageBytes = Base64.decode(currentValue, 0);

						Bitmap imageBitmap = BitmapFactory.decodeByteArray(imageBytes, 0,
								imageBytes.length);
						imageButton.setScaleType(ScaleType.CENTER_INSIDE);
						imageButton.setImageBitmap(imageBitmap);
					}

					imageButton.setId(getUniqueViewId());

					workItemElementEntry.setViewId(imageButton.getId());
					childLayout.addView(imageButton);

					AlertDialog.Builder builder = new AlertDialog.Builder(this);
					builder.setTitle("Choose action");
					builder.setItems(R.array.BinaryActions, new DialogInterface.OnClickListener() {
						public void onClick(DialogInterface dialog, int item) {

							switch (item) {
							case 0:
								Log.d(TAG, "Action: TAKE_PICTURE");
								takePicture(imageButton.getId());
								break;
							case 1:
								Log.d(TAG, "Action: SIGN");
								sign(imageButton.getId());
								break;
							default:
								break;
							}
						}
					});
					final AlertDialog alert = builder.create();

					imageButton.setOnClickListener(new OnClickListener() {

						@Override
						public void onClick(View v) {
							alert.show();
						}
					});
				} else {

					// Datatype restricted: Spinner (Combobox)
					if (workItemElementEntry.hasEnumeratedValues()) {
						Log.d(TAG, "INIT SPINNER!");
						Spinner spinner = new Spinner(this);
						ArrayAdapter<CharSequence> adapter = new ArrayAdapter<CharSequence>(this,
								android.R.layout.simple_spinner_item);
						int selectedIndex = -1;
						for (int i = 0; i < workItemElementEntry.getEnumeratedValues().size(); 
								i++) {
							String s = workItemElementEntry.getEnumeratedValues().get(i);
							adapter.add(s);
							adapter
							.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
							if (s.equals(workItemElementEntry.getValue())) {
								selectedIndex = i;
							}
						}
						spinner.setAdapter(adapter);
						if (selectedIndex >= 0) {
							spinner.setSelection(selectedIndex);
						}
						spinner.setId(getUniqueViewId());
						workItemElementEntry.setViewId(spinner.getId());
						childLayout.addView(spinner);
					} else {

						// Datatype: default -> Textbox
						ScanableEditText value = new ScanableEditText(this);
						value.setId(getUniqueViewId());

						workItemElementEntry.setViewId(value.getId());
						value.setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT,
								LayoutParams.WRAP_CONTENT));

						if (currentValue != null) {
							value.setText(currentValue);
						}
						childLayout.addView(value);

					}
				}
			} else {
				Log.d(TAG,
						workItemElementEntry.getOccursCount() + ":"
								+ workItemElementEntry.getMaxoccurs());
				// Add button for adding components if multiple occurrence is
				// permitted
				if (workItemElementEntry.getOccursCount() < workItemElementEntry.getMaxoccurs()) {

					childLayout.addView(addButtonLayout(workItemElementEntry, fatherLayout,
							childLayout));
				}
				key.setTypeface(Typeface.DEFAULT_BOLD);
				childLayout.setBackgroundResource(R.drawable.border_shape);
			}

			fatherLayout.addView(childLayout);

			// Repeat the process recursively, if there are unprocessed entries
			if (workItemElementEntry.getSubElementEntryList() != null) {
				createViewElements(childLayout, workItemElementEntry);
			}
		}
	}

	/**
	 * Adds a button layout for adding and removing view components.
	 * 
	 * @param workItemElementEntry
	 *            The work item element entry
	 * @param fatherLayout
	 *            The father layout.
	 * @param childLayout
	 *            The child layout.
	 * @return The resulting button layout.
	 */
	public View addButtonLayout(WorkItemElementEntry workItemElementEntry,
			LinearLayout fatherLayout, LinearLayout childLayout) {
		Log.d(TAG, "Adding: Add Button...");
		LinearLayout buttonLayout = new LinearLayout(this);
		buttonLayout.setOrientation(LinearLayout.HORIZONTAL);
		LayoutParams wrapContent = new LayoutParams(LayoutParams.WRAP_CONTENT,
				LayoutParams.WRAP_CONTENT);

		Button addButton = new Button(this);
		AddWorkItemElementEntryListener addListener = new AddWorkItemElementEntryListener(this,
				workItemElementEntry, fatherLayout);
		addButton.setOnClickListener(addListener);

		addButton.setLayoutParams(wrapContent);
		addButton.setText("+");
		buttonLayout.addView(addButton);

		Button removeButton = new Button(this);
		RemoveWorkItemElementEntryListener removeListener = new RemoveWorkItemElementEntryListener(
				workItemElementEntry, fatherLayout, childLayout);
		removeButton.setOnClickListener(removeListener);
		removeButton.setLayoutParams(wrapContent);
		removeButton.setText("-");
		buttonLayout.addView(removeButton);

		Log.d(TAG, workItemElementEntry.getParent().getSubElementEntryList().size() + ":"
				+ workItemElementEntry.getMinoccurs());

		if (workItemElementEntry.getParent().getSubElementEntryList().size() <= workItemElementEntry
				.getMinoccurs()) {
			removeButton.setEnabled(false);
		}

		return buttonLayout;
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.menu_workitem_started_view_edit, menu);

		return super.onCreateOptionsMenu(menu);
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {

		switch (item.getItemId()) {
		case R.id.menuWorkitemStartedViewEditCancel:
			break;
		case R.id.menuWorkitemStartedViewEditSave:
			saveAndExit();
			break;
		case R.id.menuWorkitemStartedViewEditComplete:
			complete();
			break;
		case R.id.menuWorkitemStartedViewEditScan:
			scanCode(QR_SCAN_COMPLETE);
			break;
		default:
			break;
		}

		return super.onOptionsItemSelected(item);
	}

	@Override
	public boolean onContextItemSelected(MenuItem item) {

		if (item.getTitle().equals(getString(R.string.label_scan))) {
			// IntentIntegrator.initiateScan(this);
			scanCode(QR_SCAN_SINGLE);
		}

		return super.onContextItemSelected(item);
	}

	/**
	 * Invokes the work item saving process. Collects the updated values of all
	 * view components and saves them into the mobile work item.
	 */
	private void saveWorkItem() {

		collectValues(mWorkItemElementEntryRoot.getSubElementEntryList());

		Element updatedData = WorkItemElementEntrySerializer.serialize(mWorkItemElementEntryRoot);

		Log.i(TAG, JDOMUtil.elementToString(updatedData));

		mCurrentMobileWorkItem.setUpdatedData(updatedData);

		mCurrentMobileWorkItem.setState(State.MODIFIED);
		// currentMobileWorkItem.appendSyncAction("Saved");

		try {
			mMobileWorkitemDao.update(mCurrentMobileWorkItem);
		} catch (SQLException e) {
			Log.e(TAG, "Error during updating workitem in DB ", e.fillInStackTrace());
		}
	}

	/**
	 * Invokes the work item saving process and exits the activity.
	 */
	private void saveAndExit() {

		saveWorkItem();

		Intent in = new Intent();
		setResult(EditWorkItemActivity.SAVED, in);
		finish();
	}

	/**
	 * Invokes the work item saving process and completes the activity.
	 */
	private void complete() {

		saveWorkItem();

		Intent in = new Intent();
		setResult(EditWorkItemActivity.COMPLETED, in);
		finish();
	}

	/**
	 * Collects all view component values recursively and updates the work item
	 * element entries.
	 * 
	 * @param list
	 *            The list of work item element entries to visit.
	 */
	private void collectValues(List<WorkItemElementEntry> list) {

		for (WorkItemElementEntry entry : list) {
			if (entry.getDatatype() != null) {

				String value = "";

				if (entry.getDatatype().equals("xsd:boolean")) {
					Log.d(TAG, entry.getViewId() + entry.getName() + " is checkbox");
					CheckBox checkBox = (CheckBox) findViewById(entry.getViewId());

					if (checkBox != null) {
						value = String.valueOf(checkBox.isChecked());
					}
				} else if (entry.getDatatype().equals("xsd:date")) {
					Log.d(TAG, entry.getViewId() + entry.getName() + " is date");

					DatePicker dp = (DatePicker) findViewById(entry.getViewId());

					if (dp != null) {
						String yearString = String.valueOf(dp.getYear());

						String monthString = String.valueOf(dp.getMonth() + 1);
						if (monthString.length() == 1) {
							monthString = "0" + monthString;
						}

						String dayString = String.valueOf(dp.getDayOfMonth());
						if (dayString.length() == 1) {
							dayString = "0" + dayString;
						}

						value = new StringBuilder().append(yearString).append("-")
								.append(monthString).append("-").append(dayString).toString();
					}
				} else if (entry.getDatatype().equals("xsd:base64Binary")) {

					Log.d(TAG, entry.getViewId() + entry.getName() + " is base64Binary");

					ImageButton ib = (ImageButton) findViewById(entry.getViewId());
					Log.d(TAG, entry.getViewId() + entry.getName() + "is of instance ImageButton");
					Drawable d = ib.getDrawable();

					if (d != null) {
						Bitmap bm = ((BitmapDrawable) d).getBitmap();

						if (bm != null) {
							ByteArrayOutputStream bos = new ByteArrayOutputStream();
							bm.compress(CompressFormat.JPEG, COMPRESSION_LEVEL, bos);
							byte[] bitmapdata = bos.toByteArray();

							value = Base64.encodeToString(bitmapdata, 0);
						} else {
							Log.d(TAG, "BITMAP == null");
						}

					}

				} else {
					if (entry.hasEnumeratedValues()) {
						Spinner spinner = (Spinner) findViewById(entry.getViewId());
						Log.d(TAG, entry.getViewId() + entry.getName() + "is of instance Spinner");

						if (spinner != null) {
							value = spinner.getSelectedItem().toString();
							Log.d(TAG, "Spinner-value: " + value);
						}
					} else {

						ScanableEditText editText = (ScanableEditText) findViewById(entry
								.getViewId());

						Log.d(TAG, entry.getViewId() + entry.getName() + "is of instance EditText");

						if (editText != null) {
							value = editText.getText().toString();
						}
					}
				}
				entry.setValue(value);
				Log.d(TAG, entry.getName() + ":" + value);
			}

			if (entry.getSubElementEntryList() != null) {
				collectValues(entry.getSubElementEntryList());
			}
		}
	}

	/**
	 * Return a unique view id.
	 * 
	 * @return The unique view id.
	 */
	public int getUniqueViewId() {
		return mUniqueViewId++;
	}

	/**
	 * Starts QR-Code scanning activity.
	 * 
	 * @param scanType
	 *            The scan type (explicit or implicit).
	 */
	private void scanCode(int scanType) {

		Intent intent = new Intent("com.google.zxing.client.android.SCAN");
		intent.setPackage("com.google.zxing.client.android");
		intent.putExtra("SCAN_MODE", "QR_CODE_MODE");
		startActivityForResult(intent, scanType);

	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent intent) {

		if (resultCode == RESULT_OK) {
			Log.d(TAG, "onActivityResult");

			switch (requestCode) {
			case QR_SCAN_SINGLE:
				String singleScanResults = intent.getStringExtra("SCAN_RESULT");
				if (getCurrentFocus() != null) {

					ScanableEditText set = (ScanableEditText) getCurrentFocus();
					if (set != null) {
						set.setText(singleScanResults);
					}
				}
				break;
			case QR_SCAN_COMPLETE:
				String scanResults = intent.getStringExtra("SCAN_RESULT");
				Map<String, String> scanResultMap = QRCodeDecoder.getResultsMap(scanResults);
				applyScanResults(this.mWorkItemElementEntryRoot.getSubElementEntryList(),
						scanResultMap);
				break;
			case TAKE_PICTUTE:
				Log.d(TAG, "TAKE_PICTURE_REQUEST");
				String picBase64Binary = intent.getStringExtra("BASE64_BINARY");
				int buttonViewId = intent.getIntExtra("VIEW_ID", -1);

				applyPicture(buttonViewId, picBase64Binary);

				break;
			case SIGN:
				Log.d(TAG, "SIGNING_REQUEST");
				String base64Binary = intent.getStringExtra("BASE64_BINARY");
				int viewId = intent.getIntExtra("VIEW_ID", -1);

				applyPicture(viewId, base64Binary);

				break;
			default:
				break;
			}

		}
	}

	/**
	 * Maps the QR code scan results to a matching work item element entry and
	 * updates the view component.
	 * 
	 * @param subElementEntryList
	 *            The list of work item element entries.
	 * 
	 * @param scanResultMap
	 *            The scan result map.
	 */
	private void applyScanResults(List<WorkItemElementEntry> subElementEntryList,
			Map<String, String> scanResultMap) {

		for (WorkItemElementEntry item : subElementEntryList) {
			if (scanResultMap.containsKey(item.getName())) {
				Log.d(TAG, "FOUND KEY in MAP: " + item.getName());
				item.setValue(scanResultMap.get(item.getName()));

				EditText text = (EditText) findViewById(item.getViewId());
				text.setText(item.getValue());
			}
			if (item.getSubElementEntryList() != null) {
				applyScanResults(item.getSubElementEntryList(), scanResultMap);
			}
		}
	}

	/**
	 * Starts the camera activity for taking pictures.
	 * 
	 * @param viewId
	 *            The view id to update afterwards.
	 */
	private void takePicture(int viewId) {
		Intent cameraIntent = new Intent(this, CameraActivity.class);
		cameraIntent.putExtra("VIEW_ID", viewId);
		startActivityForResult(cameraIntent, TAKE_PICTUTE);
	}

	/**
	 * Starts the SigningActivity for painting touch screen signatures.
	 * 
	 * @param viewId
	 *            The view id to update afterwards.
	 */
	private void sign(int viewId) {
		Intent signingIntent = new Intent(this, SigningActivity.class);
		signingIntent.putExtra("VIEW_ID", viewId);
		startActivityForResult(signingIntent, SIGN);
	}

	/**
	 * Applies BASE 64 image string the specified image button.
	 * 
	 * @param viewId
	 *            The view id to apply the image to.
	 * @param base64ImageString
	 *            The BASE 64 image string.
	 */
	private void applyPicture(int viewId, String base64ImageString) {

		if (viewId > -1) {
			ImageButton ib = (ImageButton) findViewById(viewId);

			if (ib != null) {
				byte[] imageBytes = Base64.decode(base64ImageString, 0);

				Bitmap imageBitmap = BitmapFactory
						.decodeByteArray(imageBytes, 0, imageBytes.length);
				ib.setImageBitmap(imageBitmap);
			} else {
				Log.d(TAG, "IB NULL");
			}
		} else {
			Log.d(TAG, "VIEW_ID = -1");
		}
	}

	/**
	 * Returns the current mobile work item.
	 * 
	 * @return The current mobile work item.
	 */
	public MobileWorkItem getCurrentMobileWorkItem() {
		return mCurrentMobileWorkItem;
	}

	/**
	 * Returns the current work item element entry root.
	 * 
	 * @return The current work item element entry root.
	 */
	public WorkItemElementEntry getWorkItemElementEntryRoot() {
		return mWorkItemElementEntryRoot;
	}
}