package de.msponer.android.yac.activities.workqueues.listener;

import java.util.ArrayList;
import java.util.List;

import android.graphics.Typeface;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.LinearLayout;
import android.widget.TextView;
import de.msponer.android.yac.R;
import de.msponer.android.yac.activities.workqueues.EditWorkItemActivity;
import de.msponer.android.yac.xml.WorkItemElementEntry;

/**
 * Onclicklistener for adding an work item element entry to a parent linear
 * layout.
 * 
 * @author msponer
 * 
 */
public class AddWorkItemElementEntryListener implements OnClickListener {

	/**
	 * The parent panel in which the view components are added.
	 */
	private LinearLayout mParentPanel;

	/**
	 * The invoking EditWorkItemActivity.
	 */
	private EditWorkItemActivity mEditWorkItemActivity;

	/**
	 * The current work item element entry.
	 */
	private WorkItemElementEntry mWorkItemElementEntry;

	/**
	 * Constructor setting the required attributs.
	 * 
	 * @param editWorkItemActivity
	 *            The invoking EditWorkItemActivity
	 * @param workItemElementEntry
	 *            The work item element entry.
	 * @param parentPanel
	 *            The parent panel.
	 */
	public AddWorkItemElementEntryListener(EditWorkItemActivity editWorkItemActivity,
			WorkItemElementEntry workItemElementEntry, LinearLayout parentPanel) {
		super();
		this.mEditWorkItemActivity = editWorkItemActivity;
		this.mWorkItemElementEntry = workItemElementEntry;
		this.mParentPanel = parentPanel;
	}

	/**
	 * {@inheritDoc} Clones the given work element entry and adds the
	 * corresponding view components to the parent panel.
	 */
	@Override
	public void onClick(View v) {

		Log.d("Add-Listener", "Adding Subpanel: " + mWorkItemElementEntry);
		WorkItemElementEntry clonedEntry = mWorkItemElementEntry.clone();
		Log.d("Listener", "clone-name: " + clonedEntry.getName());
		Log.d("Listener", "clone-parent" + clonedEntry.getParent().getName());

		List<WorkItemElementEntry> subClones = new ArrayList<WorkItemElementEntry>();
		for (WorkItemElementEntry subWIEE : clonedEntry.getSubElementEntryList()) {
			subWIEE.setParent(clonedEntry);
			subClones.add(subWIEE.clone());
		}
		clonedEntry.setSubElementEntryList(null);
		clonedEntry.setSubElementEntryList(subClones);

		clonedEntry.getParent().getSubElementEntryList().add(clonedEntry);

		LinearLayout childLayout = new LinearLayout(mEditWorkItemActivity);
		childLayout.setOrientation(LinearLayout.VERTICAL);

		// Setting key
		TextView key = new TextView(mEditWorkItemActivity);
		String currentKey = clonedEntry.getName();
		key.setText(currentKey);
		key.setId(mEditWorkItemActivity.getUniqueViewId());
		childLayout.addView(key);

		if (clonedEntry.getOccursCount() < clonedEntry.getMaxoccurs()) {

			childLayout.addView(mEditWorkItemActivity.addButtonLayout(clonedEntry, mParentPanel,
					childLayout));
		}
		key.setTypeface(Typeface.DEFAULT_BOLD);
		childLayout.setBackgroundResource(R.drawable.border_shape);
		mParentPanel.addView(childLayout);

		mEditWorkItemActivity.createViewElements(childLayout, clonedEntry);

	}

	/**
	 * Sets the parent panel.
	 * @param parentLinearLayoutPanel The parent panel.
	 */
	public void setParentPanel(LinearLayout parentLinearLayoutPanel) {
		this.mParentPanel = parentLinearLayoutPanel;
	}

	/**
	 * Returns the parent panel.
	 * @return The parent panel.
	 */
	public LinearLayout getParentPanel() {
		return mParentPanel;
	}

}
