/**
 * 
 */
package de.msponer.android.yac.activities.workqueues;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import android.app.ListActivity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnCreateContextMenuListener;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ListView;
import android.widget.TextView;

import com.j256.ormlite.android.apptools.OpenHelperManager;

import de.msponer.android.yac.R;
import de.msponer.android.yac.activities.position.WorkItemMapActivity;
import de.msponer.android.yac.adapter.LocalizedWorkItemAdapter;
import de.msponer.android.yac.adapter.SectionedWorkListAdapter;
import de.msponer.android.yac.db.QueryHelper;
import de.msponer.android.yac.model.MobileWorkItem;
import de.msponer.android.yac.util.MobileWorkItemDistanceComparator;

/**
 * Activity for localizing work items using a list view. ("Nearby work").
 * 
 * @author marcel
 */
public class LocalizedWorkListActivity extends ListActivity implements LocationListener {

	/**
	 * The activity tag.
	 */
	private static final String TAG = "LocalizedWorkListActivity";

	/**
	 * The list of lacalized mobile work items.
	 */
	private List<MobileWorkItem> mWorkitemList;

	/**
	 * The work queue list view.
	 */
	private ListView mWorkqueueListView;

	/**
	 * The reuest code.
	 */
	private int mRequestCode;

	/**
	 * The selected mobile work item.
	 */
	private MobileWorkItem mSelectedItem;

	/**
	 * The sectioned work list adapter.
	 */
	private SectionedWorkListAdapter mWorkItemAdapter;

	/**
	 * The location manager.
	 */
	private LocationManager mLocationManager;

	/**
	 * The current location.
	 */
	private Location mCurrentLocation;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);

		setContentView(R.layout.list_view_workqueue);

		mLocationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);

		mLocationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 10000, 100, this);

		// Get the last known location on start-up
		this.mCurrentLocation = mLocationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
		if (mCurrentLocation != null) {
			setTitle(mCurrentLocation.getLongitude() + ", " + mCurrentLocation.getLatitude());
		}
	}

	@Override
	protected void onResume() {
		try {
			setupList();
		} catch (Exception e) {
			e.printStackTrace();
		}

		if (this.mWorkitemList != null && this.mWorkitemList.size() > 0) {
			this.mWorkqueueListView.setOnItemClickListener(new OnItemClickListener() {
				@Override
				public void onItemClick(AdapterView<?> arg0, View view, int position, long id) {
					mSelectedItem = (MobileWorkItem) mWorkItemAdapter.getItem(position);
					startWorkItemActivity(WorkItemActivity.EDIT_INTENT);
				}
			});

			this.mWorkqueueListView
					.setOnCreateContextMenuListener(new OnCreateContextMenuListener() {

						@Override
						public void onCreateContextMenu(ContextMenu menu, View v,
								ContextMenuInfo menuInfo) {

							AdapterView.AdapterContextMenuInfo info = 
								(AdapterView.AdapterContextMenuInfo) menuInfo;

							mSelectedItem = (MobileWorkItem) mWorkItemAdapter
									.getItem(info.position);
							menu.setHeaderTitle(mSelectedItem.getID());

							String[] menuItems = getResources().getStringArray(
									R.array.MenuEntriesLocalizedWorkList);

							for (int i = 0; i < menuItems.length; i++) {
								menu.add(Menu.NONE, i, i, menuItems[i]);
							}
						}
					});
		}

		super.onResume();
	}

	@Override
	protected void onDestroy() {
		OpenHelperManager.release();
		super.onDestroy();
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		if (resultCode > 0) {
			removeSelectedWorkItem();
		}

		setProgressBarIndeterminateVisibility(false);

		super.onActivityResult(requestCode, resultCode, data);
	}

	@Override
	public boolean onContextItemSelected(MenuItem item) {

		int menuItemIndex = item.getItemId();

		switch (menuItemIndex) {
		case 0:
			// View Details
			startWorkItemActivity(null);
			break;
		case 1:
			// Show in map
			Intent mapsIntent = new Intent(this, WorkItemMapActivity.class);
			mapsIntent.putExtra("WORKITEM_TASKNAME", mSelectedItem.getTaskName());
			mapsIntent.putExtra("WORKITEM_LATITUDE", mSelectedItem.getLatitude());
			mapsIntent.putExtra("WORKITEM_LONGITUDE", mSelectedItem.getLongitude());
			startActivity(mapsIntent);
			break;
		case 2:
			
			String myLocation = "";
			
			if (mCurrentLocation != null) {
				myLocation = "saddr=" + this.mCurrentLocation.getLatitude() + ","
					+ this.mCurrentLocation.getLongitude() + "&";
			}
			
			String destination = this.mSelectedItem.getLatitude() + ","
					+ this.mSelectedItem.getLongitude();

			Intent naviIntent = new Intent(Intent.ACTION_VIEW,
					Uri.parse("http://maps.google.com/maps?" + myLocation + "daddr="
							+ destination));

			startActivity(naviIntent);
			break;
		default:
			break;
		}

		return true;
	}

	/**
	 * Sets up the localized work item list.
	 */
	private void setupList() {

		SharedPreferences sharedPreferences = getSharedPreferences(
				"de.msponer.android.yac_preferences", MODE_PRIVATE);

		String latKey = sharedPreferences.getString("prefs_latitute_key", "Latitude");
		String lonKey = sharedPreferences.getString("prefs_longitude_key", "Longitude");

		try {
			this.mWorkitemList = new QueryHelper(this)
					.queryForAllLocalizedWorkitems(lonKey, latKey);
		} catch (Exception e) {
			Log.e(TAG, "Error during DB query.");
		}

		if (mWorkitemList != null && mWorkitemList.size() > 0) {

			if (mCurrentLocation != null) {
				calcDistances();
			} else {
				Log.d(TAG, "Unknown location...");
			}

			Log.d(TAG, "Sorting WorkitemList by distance...");
			Collections.sort(this.mWorkitemList, new MobileWorkItemDistanceComparator());

			this.mWorkItemAdapter = createWorkItemAdapter();

			setListAdapter(this.mWorkItemAdapter);

			mWorkItemAdapter.notifyDataSetChanged();

			this.mWorkqueueListView = getListView();

			this.mWorkqueueListView.setTextFilterEnabled(true);
		}
	}

	/**
	 * Calculates distances between work items and current location.
	 */
	private void calcDistances() {

		for (MobileWorkItem workitem : this.mWorkitemList) {

			if (workitem.getLongitude() != null && workitem.getLongitude().length() > 0
					&& workitem.getLatitude() != null && workitem.getLatitude().length() > 0) {

				Location loc = new Location("WorkItemLocationInformation");

				loc.setLongitude(Double.parseDouble(workitem.getLongitude()));
				loc.setLatitude(Double.parseDouble(workitem.getLatitude()));

				float distance = this.mCurrentLocation.distanceTo(loc);
				Log.d(TAG, "Floatdist: " + distance);
				Log.d(TAG, loc.getLongitude() + ", " + loc.getLatitude());
				Log.d(TAG, mCurrentLocation.getLongitude() + "; " + mCurrentLocation.getLatitude());

				Log.d(TAG, workitem.getID() + " distance: " + distance);

				workitem.setDistanceToPosition(distance);
			}
		}
	}

	/**
	 * Removes the selected work item for the list.
	 */
	public void removeSelectedWorkItem() {

		this.mWorkitemList.remove(mSelectedItem);

		this.mWorkItemAdapter = createWorkItemAdapter();
		this.setListAdapter(this.mWorkItemAdapter);

		if (mWorkItemAdapter != null) {
			this.mWorkItemAdapter.notifyDataSetChanged();
		}

		this.mSelectedItem = null;
	}

	/**
	 * Initializes the sectioned work list adapter and return it.
	 * 
	 * @return The sectioned work list adapter.
	 */
	public SectionedWorkListAdapter createWorkItemAdapter() {

		if (this.mWorkitemList == null || this.mWorkitemList.size() <= 0) {
			return null;
		}

		this.mWorkItemAdapter = new SectionedWorkListAdapter() {
			@Override
			protected View getHeaderView(String caption, int index, View convertView,
					ViewGroup parent) {
				TextView result = (TextView) convertView;

				if (convertView == null) {
					result = (TextView) getLayoutInflater()
							.inflate(R.layout.list_view_header, null);
				}

				result.setText(caption);

				return (result);
			}
		};

		List<List<MobileWorkItem>> sectionLists = new ArrayList<List<MobileWorkItem>>();

		for (int i = 0; i < 6; i++) {
			sectionLists.add(new ArrayList<MobileWorkItem>());
		}

		for (int i = 0; i < this.mWorkitemList.size(); i++) {
			MobileWorkItem mwi = this.mWorkitemList.get(i);

			int sectionIndex = -1;

			if (mwi.getDistanceToPosition() < 0) {
				sectionIndex = 5;
			} else if (mwi.getDistanceToPosition() < 1000) {
				sectionIndex = 0;
			} else if (mwi.getDistanceToPosition() < 10000) {
				sectionIndex = 1;
			} else if (mwi.getDistanceToPosition() < 100000) {
				sectionIndex = 2;
			} else if (mwi.getDistanceToPosition() < 1000000) {
				sectionIndex = 3;
			} else {
				sectionIndex = 4;
			}

			sectionLists.get(sectionIndex).add(mwi);
		}

		for (int i = 0; i < sectionLists.size(); i++) {
			if (sectionLists.get(i) != null) {
				List<MobileWorkItem> list = sectionLists.get(i);
				if (list.size() > 0) {
					mWorkItemAdapter.addSection(getHeaderTextForIndex(i),
							new LocalizedWorkItemAdapter(this, R.layout.list_item_localized_queue,
									list));
				}
			}
		}

		return mWorkItemAdapter;
	}

	/**
	 * Returns the header text to a corresponding index.
	 * 
	 * @param index
	 *            The index.
	 * @return The header text.
	 */
	private String getHeaderTextForIndex(int index) {

		if (index == 0) {
			return "Location within 1km";
		} else if (index == 1) {
			return "Location within 10km";
		} else if (index == 2) {
			return "Location within 100km";
		} else if (index == 3) {
			return "Location within 1000km";
		} else if (index == 4) {
			return "Location distance more than 1000km";
		}
		return "Unknown location distance";
	}

	/**
	 * Starts the work item activity.
	 * 
	 * @param intentAction
	 *            The intent action to invoke.
	 */
	public void startWorkItemActivity(String intentAction) {

		Intent workItemIntent = new Intent(this, WorkItemActivity.class);

		Log.d(TAG, "Selected WorkItem-ID: " + this.mSelectedItem.getID());
		Log.d(TAG, "Workitem Intent-Action: " + intentAction);
		workItemIntent.putExtra("PRIMARY_KEY", this.mSelectedItem.getPrimaryKey());

		if (intentAction != null && intentAction.length() > 0) {
			workItemIntent.setAction(intentAction);
		}

		if (getParent() != null) {
			getParent().setProgressBarIndeterminateVisibility(true);
		} else {
			setProgressBarIndeterminateVisibility(true);
		}

		startActivityForResult(workItemIntent, this.mRequestCode);
	}

	@Override
	public void onLocationChanged(Location location) {

		Log.d(TAG, "onLocationChanged: " + location.getLongitude() + ", " + location.getLatitude());
		this.mCurrentLocation = location;

		if (mCurrentLocation != null) {
			setTitle(mCurrentLocation.getLongitude() + ", " + mCurrentLocation.getLatitude());
		}

		setupList();
	}

	@Override
	public void onProviderDisabled(String provider) {

	}

	@Override
	public void onProviderEnabled(String provider) {

	}

	@Override
	public void onStatusChanged(String provider, int status, Bundle extras) {

	}
}
