/**
 * 
 */
package de.msponer.android.yac.activities.workqueues;

import android.content.Intent;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnCreateContextMenuListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import de.msponer.android.yac.R;
import de.msponer.android.yac.model.MobileWorkItem;

/**
 * Activity that displays a work item search result list. Used after search
 * operation and QR-Code scan.
 * 
 * @author msponer
 * 
 */
public class SearchResultListActivity extends WorkListActivity {

	/**
	 * The application tag.
	 */
	@SuppressWarnings("unused")
	private static final String TAG = "ScanResultListActivity";

	@Override
	protected void onCreate(Bundle savedInstanceState) {

		Intent callingIntent = getIntent();

		String searchPattern = callingIntent.getExtras().getString("SEARCH_PATTERN");

		setSearchPattern(searchPattern);
		setTitle("Scan-Results for: " + searchPattern.replace("%", ""));
		super.onCreate(savedInstanceState);
	}

	@Override
	protected void onResume() {
		super.onResume();

		if (getWorkitemList() != null && getWorkitemList().size() > 0) {
			getWorkqueueListView().setOnItemClickListener(new OnItemClickListener() {
				@Override
				public void onItemClick(AdapterView<?> arg0, View view, int position, long id) {

					setSelectedItem((MobileWorkItem) getWorkItemAdapter().getItem(position));
					startWorkItemActivity(WorkItemActivity.EDIT_INTENT);
				}
			});

			getWorkqueueListView().setOnCreateContextMenuListener(
					new OnCreateContextMenuListener() {

						@Override
						public void onCreateContextMenu(ContextMenu menu, View v,
								ContextMenuInfo menuInfo) {

							AdapterView.AdapterContextMenuInfo info = 
								(AdapterView.AdapterContextMenuInfo) menuInfo;

							MobileWorkItem selItem = (MobileWorkItem) getWorkItemAdapter().getItem(
									info.position);
							menu.setHeaderTitle(selItem.getID());
							setSelectedItem(selItem);

							String[] menuItems = getResources().getStringArray(
									R.array.MenuEntriesScanResultList);

							for (int i = 0; i < menuItems.length; i++) {
								menu.add(Menu.NONE, i, i, menuItems[i]);
							}
						}
					});
		}
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		if (resultCode > 0) {
			removeSelectedWorkItem();
		}

		setProgressBarIndeterminateVisibility(false);

		super.onActivityResult(requestCode, resultCode, data);
	}

	@Override
	public boolean onContextItemSelected(MenuItem item) {

		int menuItemIndex = item.getItemId();

		switch (menuItemIndex) {
		case 0: 
			// View Details
			startWorkItemActivity(null);
			break;
		default:
			break;
		}

		return true;
	}

}
