package de.msponer.android.yac.activities.workqueues;

import android.content.Intent;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnCreateContextMenuListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import de.msponer.android.yac.R;
import de.msponer.android.yac.model.MobileWorkItem;

/**
 * Activity for allocated work items. View is embedded in a tab host.
 * 
 * @author msponer
 * 
 */
public class TabAllocatedActivity extends WorkListActivity {

	/**
	 * The activity tag.
	 */
	@SuppressWarnings("unused")
	private static final String TAG = "TabAllocatedActivity";

	@Override
	protected void onCreate(Bundle savedInstanceState) {

		setSearchPattern("Allocated");

		super.onCreate(savedInstanceState);
	}

	@Override
	protected void onResume() {
		super.onResume();

		if (getWorkitemList() != null 
					&& getWorkitemList().size() > 0) {
			getWorkqueueListView().setOnItemClickListener(new OnItemClickListener() {
				@Override
				public void onItemClick(AdapterView<?> arg0, View view, int position, long id) {

					setSelectedItem((MobileWorkItem) getWorkItemAdapter().getItem(position));
					startWorkItemActivity(WorkItemActivity.EDIT_INTENT);
				}
			});

			getWorkqueueListView().setOnCreateContextMenuListener(
					new OnCreateContextMenuListener() {

						@Override
						public void onCreateContextMenu(ContextMenu menu, View v,
								ContextMenuInfo menuInfo) {

							AdapterView.AdapterContextMenuInfo info = 
								(AdapterView.AdapterContextMenuInfo) menuInfo;

							MobileWorkItem selItem = (MobileWorkItem) getWorkItemAdapter().getItem(
									info.position);
							menu.setHeaderTitle(selItem.getID());
							setSelectedItem(selItem);

							String[] menuItems = getResources().getStringArray(
									R.array.MenuEntriesTabAllocated);

							for (int i = 0; i < menuItems.length; i++) {
								menu.add(Menu.NONE, i, i, menuItems[i]);
							}
						}
					});
		}
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

//		switch (resultCode) {
//		case WorkItemActivity.START:
//			startWorkItem();
//			break;
//		case WorkItemActivity.DEALLOCATE:
//			deallocate();
//			break;
//		case WorkItemActivity.DELEGATE:
//			delegate();
//			break;
//		case WorkItemActivity.SKIP:
//			skip();
//			break;
//		case WorkItemActivity.PILE:
//			pile();
//			break;
//		default:
//			break;
//		}

		removeSelectedWorkItem();
		
		getParent().setProgressBarIndeterminateVisibility(false);

		super.onActivityResult(requestCode, resultCode, data);
	}

	@Override
	public boolean onContextItemSelected(MenuItem item) {

		int menuItemIndex = item.getItemId();

		switch (menuItemIndex) {
		case 0:
			// View Details
			startWorkItemActivity(null);
			break;
		case 1:
			// Start
			startWorkItemActivity(WorkItemActivity.START_INTENT);
			break;
		case 2:
			// Deallocate
			startWorkItemActivity(WorkItemActivity.DEALLOCATE_INTENT);
			break;
		case 3:
			// Skip
			startWorkItemActivity(WorkItemActivity.SKIP_INTENT);
			break;
		default:
			break;
		}

		return true;
	}
	
	/**
	 * Starts the work item and removes it from the current list.
	 */
	@SuppressWarnings("unused")
	private void startWorkItem() {
		removeSelectedWorkItem();
	}

	/**
	 * Deallocates the work item and removes it from the current list.
	 */
	@SuppressWarnings("unused")
	private void deallocate() {
		removeSelectedWorkItem();
	}

	/**
	 * Delegates the work item and removes it from the current list.
	 */
	@SuppressWarnings("unused")
	private void delegate() {
		// TODO Auto-generated method stub
	}

	/**
	 * Skips the work item and removes it from the current list.
	 */
	@SuppressWarnings("unused")
	private void skip() {
		removeSelectedWorkItem();
	}

	/**
	 * Piles the work item and removes it from the current list.
	 */
	@SuppressWarnings("unused")
	private void pile() {
		removeSelectedWorkItem();
	}
}
