/**
 * 
 */
package de.msponer.android.yac.activities.workqueues;

import android.content.Intent;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnCreateContextMenuListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import de.msponer.android.yac.R;
import de.msponer.android.yac.model.MobileWorkItem;

/**
 * Activity for offered work items. View is embedded in a tab host.
 * @author msponer
 * 
 */
public class TabOfferedActivity extends WorkListActivity {

	/**
	 * The activity tag.
	 */
	@SuppressWarnings("unused")
	private static final String TAG = "TabOfferedActivity";

	@Override
	protected void onCreate(Bundle savedInstanceState) {

		setSearchPattern("Offered");

		super.onCreate(savedInstanceState);
	}

	@Override
	protected void onResume() {
		super.onResume();

		if (getWorkitemList() != null && getWorkitemList().size() > 0) {
			getWorkqueueListView().setOnItemClickListener(new OnItemClickListener() {
				@Override
				public void onItemClick(AdapterView<?> arg0, View view, int position, long id) {
					
					setSelectedItem((MobileWorkItem) getWorkItemAdapter().getItem(position));
					startWorkItemActivity(WorkItemActivity.EDIT_INTENT);
				}
			});

			getWorkqueueListView().setOnCreateContextMenuListener(
					new OnCreateContextMenuListener() {

						@Override
						public void onCreateContextMenu(ContextMenu menu, View v,
								ContextMenuInfo menuInfo) {

							AdapterView.AdapterContextMenuInfo info = 
								(AdapterView.AdapterContextMenuInfo) menuInfo;

							MobileWorkItem selItem = (MobileWorkItem) getWorkItemAdapter().getItem(
									info.position);
							menu.setHeaderTitle(selItem.getID());
							setSelectedItem(selItem);

							String[] menuItems = getResources().getStringArray(
									R.array.MenuEntriesTabOffered);

							for (int i = 0; i < menuItems.length; i++) {
								menu.add(Menu.NONE, i, i, menuItems[i]);
							}
						}
					});
		}
	}

	@Override
	public boolean onContextItemSelected(MenuItem item) {

		int menuItemIndex = item.getItemId();

		switch (menuItemIndex) {
		case 0:
			// View Details
			startWorkItemActivity(null);
			break;
		case 1:
			// Accept offer
			startWorkItemActivity(WorkItemActivity.ACCEPT_OFFER_INTENT);
			break;
		case 2:
			// Accept and start
			startWorkItemActivity(WorkItemActivity.ACCEPT_START_INTENT);
			break;
		default:
			break;
		}

		return true;
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

//		switch (resultCode) {
//		case WorkItemActivity.ACCEPT_OFFER:
////			acceptOffer();
//			break;
//		case WorkItemActivity.ACCEPT_AND_START:
////			acceptAndStartOffer();
//			break;
//		case WorkItemActivity.CHAIN:
////			chain();
//			break;
//		default:
//			break;
//		}

		removeSelectedWorkItem();
		
		getParent().setProgressBarIndeterminateVisibility(false);

		super.onActivityResult(requestCode, resultCode, data);
	}
	
	/**
	 * Accepts the offered work item and removes it from the current list.
	 */
	@SuppressWarnings("unused")
	private void acceptOffer() {

		removeSelectedWorkItem();
	}

	/**
	 * Accepts the offered work item, starts it and removes it from the current list.
	 */
	@SuppressWarnings("unused")
	private void acceptAndStartOffer() {

		removeSelectedWorkItem();
	}

	/**
	 * Chains the work item and removes it from the current list.
	 */
	@SuppressWarnings("unused")
	private void chain() {

		removeSelectedWorkItem();
	}

}
