/**
 * 
 */
package de.msponer.android.yac.activities.workqueues;

import android.content.Intent;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnCreateContextMenuListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import de.msponer.android.yac.R;
import de.msponer.android.yac.model.MobileWorkItem;

/**
 * Activity for started work items. View is embedded in a tab host.
 * 
 * @author marcel
 * 
 */
public class TabStartedActivity extends WorkListActivity {

	/**
	 * The activity tag.
	 */
	@SuppressWarnings("unused")
	private static final String TAG = "TabStartedActivity";

	@Override
	protected void onCreate(Bundle savedInstanceState) {

		setSearchPattern("Started");
		super.onCreate(savedInstanceState);
	}

	@Override
	protected void onResume() {
		super.onResume();

		if (getWorkitemList() != null && getWorkitemList().size() > 0) {
			getWorkqueueListView().setOnItemClickListener(new OnItemClickListener() {
				@Override
				public void onItemClick(AdapterView<?> arg0, View view, int position, long id) {
					setSelectedItem((MobileWorkItem) getWorkItemAdapter().getItem(position));
					startWorkItemActivity(WorkItemActivity.EDIT_INTENT);
				}
			});

			getWorkqueueListView().setOnCreateContextMenuListener(
					new OnCreateContextMenuListener() {

						@Override
						public void onCreateContextMenu(ContextMenu menu, View v,
								ContextMenuInfo menuInfo) {

							AdapterView.AdapterContextMenuInfo info = 
								(AdapterView.AdapterContextMenuInfo) menuInfo;

							MobileWorkItem selItem = (MobileWorkItem) getWorkItemAdapter().getItem(
									info.position);
							menu.setHeaderTitle(selItem.getID());
							setSelectedItem(selItem);

							String[] menuItems = getResources().getStringArray(
									R.array.MenuEntriesTabStarted);

							for (int i = 0; i < menuItems.length; i++) {
								menu.add(Menu.NONE, i, i, menuItems[i]);
							}
						}
					});
		}
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		switch (resultCode) {
		case WorkItemActivity.SUSPEND:
			suspend();
			break;
		case WorkItemActivity.REALLOCATE_SL:
			reallocateSL();
			break;
		case WorkItemActivity.REALLOCATE_SF:
			reallocateSF();
			break;
		case WorkItemActivity.NEW_INSTANCE:
			newInstance();
			break;
		case WorkItemActivity.COMPLETE:
			complete();
			break;
		case WorkItemActivity.UPDATE:
			break;
		default:
			break;
		}

		getParent().setProgressBarIndeterminateVisibility(false);

		super.onActivityResult(requestCode, resultCode, data);
	}

	@Override
	public boolean onContextItemSelected(MenuItem item) {

		int menuItemIndex = item.getItemId();

		switch (menuItemIndex) {
		case 0:
			// View Details
			startWorkItemActivity(null);
			break;
		case 1:
			// Edit
			startWorkItemActivity(WorkItemActivity.EDIT_INTENT);
			break;
		case 2:
			// Suspend
			startWorkItemActivity(WorkItemActivity.SUSPEND_INTENT);
			break;
		case 3:
			// COMPLETE
			startWorkItemActivity(WorkItemActivity.COMPLETE_INTENT);
			break;
		default:
			break;
		}

		return true;
	}

	/**
	 * Suspends the work item and removes it from the current list.
	 */
	private void suspend() {

		removeSelectedWorkItem();
	}

	/**
	 * Reallocates (sl) the work item and removes it from the current list.
	 */
	private void reallocateSL() {
		// TODO Auto-generated method stub
	}

	/**
	 * Reallocates (sf) the work item and removes it from the current list.
	 */
	private void reallocateSF() {
		// TODO Auto-generated method stub
	}

	/**
	 * Instantiates a new work item.
	 */
	private void newInstance() {
		// TODO Auto-generated method stub
	}

	/**
	 * Completes the work item and removes it from the current list.
	 */
	private void complete() {

		removeSelectedWorkItem();
	}
}
