/**
 * 
 */
package de.msponer.android.yac.activities.workqueues;

import android.app.TabActivity;
import android.content.Intent;
import android.os.Bundle;
import android.view.Window;
import android.widget.TabHost;
import android.widget.TabHost.TabSpec;
import de.msponer.android.yac.R;

/**
 * Tab host activity for the work queues.
 * 
 * @author msponer
 * 
 */
public class WorkQueuesTabHostActivity extends TabActivity {

	/**
	 * The activity tag.
	 */
	@SuppressWarnings("unused")
	private static final String TAG = "WorkQueuesTabHostActivity";

	/**
	 * Offered tag string.
	 */
	private static final String TAB_OFFERED = "tabOffered";
	
	/**
	 * Allocated tab string.
	 */
	private static final String TAB_ALLOCATED = "tabAllocated";
	
	/**
	 * Started tab string.
	 */
	private static final String TAB_STARTED = "tabStarted";
	
	/**
	 * Suspended tab string.
	 */
	private static final String TAB_SUSPENDED = "tabSuspended";

	/**
	 * Intent to invoke the OfferedTab.
	 */
	private Intent mOfferedTabIntent;
	
	/**
	 * Intent to invoke the AllocatedTab.
	 */
	private Intent mAllocatedTabIntent;
	
	/**
	 * Intent to invoke the StartedTab.
	 */
	private Intent mStartedTabIntent;
	
	/**
	 * Intent to invoke the SuspendedTab.
	 */
	private Intent mSuspendedTabIntent;

	/**
	 * The offered TabSpec.
	 */
	private TabSpec mOfferedTabSpec;
	
	/**
	 * The allocated TabSpec.
	 */
	private TabSpec mAllocatedTabSpec;
	
	/**
	 * The started TabSpec.
	 */
	private TabSpec mStartedTabSpec;
	
	/**
	 * The suspended TabSpec.
	 */
	private TabSpec mSuspendedTabSpec;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);

		setContentView(R.layout.work_queues);

		TabHost tabHost = (TabHost) findViewById(android.R.id.tabhost);

		mOfferedTabSpec = tabHost.newTabSpec(TAB_OFFERED);
		mAllocatedTabSpec = tabHost.newTabSpec(TAB_ALLOCATED);
		mStartedTabSpec = tabHost.newTabSpec(TAB_STARTED);
		mSuspendedTabSpec = tabHost.newTabSpec(TAB_SUSPENDED);

		this.mOfferedTabIntent = new Intent(this, TabOfferedActivity.class);
		this.mAllocatedTabIntent = new Intent(this, TabAllocatedActivity.class);
		this.mStartedTabIntent = new Intent(this, TabStartedActivity.class);
		this.mSuspendedTabIntent = new Intent(this, TabSuspendedActivity.class);

		setupTabIndicators();

		/* Add tabSpec to the TabHost to display. */
		tabHost.addTab(mOfferedTabSpec);
		tabHost.addTab(mAllocatedTabSpec);
		tabHost.addTab(mStartedTabSpec);
		tabHost.addTab(mSuspendedTabSpec);

	}

	/**
	 * Sets up to tab indicators.
	 */
	private void setupTabIndicators() {

		mOfferedTabSpec.setIndicator(getString(R.string.label_tab_offered),
				getResources().getDrawable(R.drawable.ic_tab_offered)).setContent(
				this.mOfferedTabIntent);

		mAllocatedTabSpec.setIndicator(getString(R.string.label_tab_allocated),
				getResources().getDrawable(R.drawable.ic_tab_allocated)).setContent(
				this.mAllocatedTabIntent);

		mStartedTabSpec.setIndicator(getString(R.string.label_tab_started),
				getResources().getDrawable(R.drawable.ic_tab_started)).setContent(
				this.mStartedTabIntent);

		mSuspendedTabSpec.setIndicator(getString(R.string.label_tab_suspended),
				getResources().getDrawable(R.drawable.ic_tab_suspended)).setContent(
				this.mSuspendedTabIntent);

	}
}
