/**
 * 
 */
package de.msponer.android.yac.adapter;

import java.util.List;

import android.content.Context;
import android.graphics.Color;
import android.text.format.DateFormat;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.TextView;
import de.msponer.android.yac.R;
import de.msponer.android.yac.model.MobileWorkItem;
import de.msponer.android.yac.model.State;

/**
 * Adapter class for localized work items.
 * 
 * @author msponer
 * 
 */
public class LocalizedWorkItemAdapter extends ArrayAdapter<MobileWorkItem> {
	/**
	 * The localized work item list.
	 */
	private List<MobileWorkItem> mWorkitemList;

	/**
	 * Constructor setting the required fields.
	 * 
	 * @param context
	 *            The application context.
	 * @param textViewResourceId
	 *            The view id.
	 * @param workitemList
	 *            The localized work item list.
	 */
	public LocalizedWorkItemAdapter(Context context, int textViewResourceId,
			List<MobileWorkItem> workitemList) {

		super(context, textViewResourceId, workitemList);
		this.setWorkitemList(workitemList);
	}

	/**
	 * {@inheritDoc} The view component containing the localized work item
	 * information.
	 */
	@Override
	public View getView(int position, View convertView, ViewGroup parent) {

		View v = convertView;

		if (v == null) {
			LayoutInflater vi = (LayoutInflater) getContext().getSystemService(
					Context.LAYOUT_INFLATER_SERVICE);
			v = vi.inflate(R.layout.list_item_localized_queue, null);
		}

		MobileWorkItem mwi = getWorkitemList().get(position);
		if (mwi != null) {
			TextView tt = (TextView) v.findViewById(R.id.textViewListItemTop);
			TextView mt = (TextView) v.findViewById(R.id.textViewListItemMid);
			TextView bt = (TextView) v.findViewById(R.id.textViewListItemBottom);

			if (tt != null) {

				tt.setText(mwi.getTaskName() + " (" + mwi.getCaseID() + ") - "
						+ mwi.getResourceStatus());

				if (mwi.getState() == State.UNMODIFIED) {
					// Green
					tt.setTextColor(Color.rgb(102, 153, 0));
				} else {
					// Red
					tt.setTextColor(Color.rgb(204, 0, 0));
				}
			}

			if (mt != null) {
				if (mwi.getLatitude() != null && mwi.getLatitude().length() > 0
						&& mwi.getLongitude() != null && mwi.getLatitude().length() > 0) {

					String mText = "Longitude: " + mwi.getLongitude() + ", Latitude: "
							+ mwi.getLatitude();

					if (mwi.getDistanceToPosition() >= 0) {
						String distance = String.valueOf(mwi.getDistanceToPosition() / 1000);
						if (distance.contains(".")
								&& (distance.indexOf(".") + 3) < distance.length()) {
							distance = distance.substring(0, distance.indexOf(".") + 3);

						}
						mText += ", Distance: " + distance + "km";

					}

					mt.setText(mText);
				} else {
					mt.setText("Unknown location");
					mt.setTextColor(Color.rgb(204, 0, 0));
				}
			}

			if (bt != null) {
				bt.setText(DateFormat.getDateFormat(getContext()).format(mwi.getEnablementDate())
						+ " "
						+ DateFormat.getTimeFormat(getContext()).format(mwi.getEnablementDate()));
			}
		}
		return v;
	}

	/**
	 * Rounds a given value.
	 * 
	 * @param val
	 *            The value to round.
	 * @param sca
	 *            The scale.
	 * @return The rounded valued.
	 */
	public static double round(double val, int sca) {

		double s = Math.pow(10, sca);
		return Math.round(val * s) / s;
	}

	/**
	 * Sets the current localized work item list.
	 * 
	 * @param workitemList
	 *            The localized work item list.
	 */
	public void setWorkitemList(List<MobileWorkItem> workitemList) {
		this.mWorkitemList = workitemList;
	}

	/**
	 * Returns the current localized work item list.
	 * 
	 * @return The current work item list.
	 */
	public List<MobileWorkItem> getWorkitemList() {
		return mWorkitemList;
	}
}
