package de.msponer.android.yac.adapter;

import java.util.ArrayList;
import java.util.List;

import org.jdom.Element;
import org.yawlfoundation.yawl.util.JDOMUtil;

import android.content.Context;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AbsListView;
import android.widget.BaseExpandableListAdapter;
import android.widget.TextView;

/**
 * A work item adapter for an expandable data list.
 * 
 * @author msponer
 * 
 */
public class WorkItemDataExpendableListAdapter extends BaseExpandableListAdapter {

	/**
	 * The application context.
	 */
	private Context mContext;

	/**
	 * The work item XML data root.
	 */
	@SuppressWarnings("unused")
	private Element mWorkItemXmlDataRoot;

	/**
	 * The list of groups.
	 */
	private List<String> mGroups;

	/**
	 * The list of child lists.
	 */
	private List<List<String>> mChildren;

	/**
	 * Constructor setting the required fields.
	 * 
	 * @param c
	 *            The application context.
	 * @param workItemXmlDataRoot
	 *            The work item XML data root.
	 */
	public WorkItemDataExpendableListAdapter(Context c, Element workItemXmlDataRoot) {
		this.mContext = c;
		this.mWorkItemXmlDataRoot = workItemXmlDataRoot;

		@SuppressWarnings("unchecked")
		List<Element> xmlChildren = workItemXmlDataRoot.getChildren();

		mGroups = new ArrayList<String>();
		mChildren = new ArrayList<List<String>>();

		for (int i = 0; i < xmlChildren.size(); i++) {

			mGroups.add(xmlChildren.get(i).getName());

			List<String> childList = new ArrayList<String>();

			if (xmlChildren.get(i).getChildren().size() > 0) {

				@SuppressWarnings("unchecked")
				List<Element> subChildren = xmlChildren.get(i).getChildren();
				for (int j = 0; j < subChildren.size(); j++) {
					if (subChildren.get(j).getChildren().size() > 0) {
						childList.add(JDOMUtil.elementToString(subChildren.get(j)));
					} else {
						childList.add(subChildren.get(j).getName() + ": "
								+ subChildren.get(j).getValue());
					}
				}
			} else {
				childList.add(xmlChildren.get(i).getValue());
			}
			mChildren.add(childList);
		}
	}

	@Override
	public Object getChild(int groupPosition, int childPosition) {
		return mChildren.get(groupPosition).get(childPosition);
	}

	@Override
	public long getChildId(int groupPosition, int childPosition) {
		return childPosition;
	}

	@Override
	public int getChildrenCount(int groupPosition) {
		return mChildren.get(groupPosition).size();
	}

	/**
	 * Returns a generic text view.
	 * @return A generic text view.
	 */
	public TextView getGenericView() {
		// Layout parameters for the ExpandableListView
		AbsListView.LayoutParams lp = new AbsListView.LayoutParams(
				ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);

		TextView textView = new TextView(this.mContext);
		textView.setLayoutParams(lp);
		textView.setGravity(Gravity.CENTER_VERTICAL | Gravity.LEFT);
		textView.setPadding(36, 10, 0, 10);
		return textView;
	}

	@Override
	public View getChildView(int groupPosition, int childPosition, boolean isLastChild,
			View convertView, ViewGroup parent) {
		TextView textView = getGenericView();
		textView.setText(getChild(groupPosition, childPosition).toString());
		return textView;
	}

	@Override
	public Object getGroup(int groupPosition) {
		return mGroups.get(groupPosition);
	}

	@Override
	public int getGroupCount() {
		return mGroups.size();
	}

	@Override
	public long getGroupId(int groupPosition) {
		return groupPosition;
	}

	@Override
	public View getGroupView(int groupPosition, boolean isExpanded, View convertView,
			ViewGroup parent) {
		TextView textView = getGenericView();
		textView.setText(getGroup(groupPosition).toString());
		textView.setPadding(50, 10, 0, 10);
		return textView;
	}
	
	@Override
	public boolean isChildSelectable(int groupPosition, int childPosition) {
		return true;
	}

	@Override
	public boolean hasStableIds() {
		return true;
	}
}
