/**
 * 
 */
package de.msponer.android.yac.adapter;

import java.util.List;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.TextView;
import de.msponer.android.yac.R;
import de.msponer.android.yac.model.MobileWorkItem;

/**
 * An adapter class for work item to sync.
 * 
 * @author msponer
 * 
 */
public class WorkItemSyncAdapter extends ArrayAdapter<MobileWorkItem> {

	/**
	 * The list of work item to sync.
	 */
	private List<MobileWorkItem> mWorkitemList;

	/**
	 * Constructor setting the required fields.
	 * @param context The application context.
	 * @param textViewResourceId The view id.
	 * @param workitemList The work item list.
	 */
	public WorkItemSyncAdapter(Context context, int textViewResourceId,
			List<MobileWorkItem> workitemList) {

		super(context, textViewResourceId, workitemList);
		this.setWorkitemList(workitemList);
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {

		View v = convertView;

		if (v == null) {
			LayoutInflater vi = (LayoutInflater) getContext().getSystemService(
					Context.LAYOUT_INFLATER_SERVICE);
			v = vi.inflate(R.layout.list_item_queues, null);
		}

		MobileWorkItem mwi = getWorkitemList().get(position);
		if (mwi != null) {
			TextView tt = (TextView) v.findViewById(R.id.textViewListItemTop);
			TextView bt = (TextView) v.findViewById(R.id.textViewListItemBottom);

			if (tt != null) {
				tt.setText(mwi.getID());
			}

			if (bt != null) {
				String queueString = mwi.getSyncQueue();
				if (queueString.endsWith(",")) {
					queueString = queueString.substring(0, queueString.length() - 1);
				}
				bt.setText(queueString);
			}
		}
		return v;
	}

	/**
	 * Sets the work item list.
	 * @param workitemList The work item list to set.
	 */
	public void setWorkitemList(List<MobileWorkItem> workitemList) {
		this.mWorkitemList = workitemList;
	}

	/**
	 * Returns the work item list.
	 * @return The work item list.
	 */
	public List<MobileWorkItem> getWorkitemList() {
		return mWorkitemList;
	}

}
