/**
 * 
 */
package de.msponer.android.yac.db;

import java.sql.SQLException;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

import com.j256.ormlite.android.apptools.OrmLiteSqliteOpenHelper;
import com.j256.ormlite.dao.Dao;
import com.j256.ormlite.support.ConnectionSource;
import com.j256.ormlite.table.TableUtils;

import de.msponer.android.yac.model.MobileWorkItem;

/**
 * ORM-Lite Database Helper class. For details see ORMLite manual.
 * 
 * @author msponer
 * 
 */
public class DatabaseHelper extends OrmLiteSqliteOpenHelper {

	/**
	 * Name of the database file.
	 */
	private static final String DATABASE_NAME = "ymc.db";

	/**
	 * The database version.
	 */
	private static final int DATABASE_VERSION = 1;

	/**
	 * The DAO object we use to access the MobileWorkItem table.
	 */
	private Dao<MobileWorkItem, Integer> mMobileWorkItemDao = null;

	/**
	 * Constructor setting the required parameters.
	 * 
	 * @param context
	 *            The application context.
	 */
	public DatabaseHelper(Context context) {
		super(context, DATABASE_NAME, null, DATABASE_VERSION);
	}

	/**
	 * This is called when the database is first created. Usually you should
	 * call createTable statements here to create the tables that will store
	 * your data.
	 * 
	 * @param db
	 *            The SQLite db.
	 * @param connectionSource
	 *            The connection source.
	 */
	@Override
	public void onCreate(SQLiteDatabase db, ConnectionSource connectionSource) {
		try {
			Log.i(DatabaseHelper.class.getName(), "onCreate");
			TableUtils.createTable(connectionSource, MobileWorkItem.class);

			// here we try inserting data in the on-create as a test
			Dao<MobileWorkItem, Integer> dao = getMobileWorkItemDao();
			long millis = System.currentTimeMillis();
			MobileWorkItem mwi = new MobileWorkItem();
			dao.create(mwi);
			Log.i(DatabaseHelper.class.getName(), "created new entry in onCreate: " + millis);
		} catch (SQLException e) {
			Log.e(DatabaseHelper.class.getName(), "Can't create database", e);
			throw new RuntimeException(e);
		}
	}

	/**
	 * This is called when your application is upgraded and it has a higher
	 * version number. This allows you to adjust the various data to match the
	 * new version number.
	 * 
	 * @param db
	 *            The SQLite db.
	 * @param connectionSource
	 *            The connection source.
	 * @param oldVersion
	 *            The old db version.
	 * @param newVersion
	 *            The new db version.
	 */
	@Override
	public void onUpgrade(SQLiteDatabase db, ConnectionSource connectionSource, int oldVersion,
			int newVersion) {
		try {
			Log.i(DatabaseHelper.class.getName(), "onUpgrade");
			TableUtils.dropTable(connectionSource, MobileWorkItem.class, true);
			// after we drop the old databases, we create the new ones
			onCreate(db, connectionSource);
		} catch (SQLException e) {
			Log.e(DatabaseHelper.class.getName(), "Can't drop databases", e);
			throw new RuntimeException(e);
		}
	}

	/**
	 * Returns the Database Access Object (DAO) for our MobileWorkItem class. It
	 * will create it or just give the cached value.
	 * 
	 * @return The mobile work item dao.
	 * @throws SQLException Error during query.
	 */
	public Dao<MobileWorkItem, Integer> getMobileWorkItemDao() throws SQLException {
		if (mMobileWorkItemDao == null) {
			mMobileWorkItemDao = getDao(MobileWorkItem.class);
		}
		return mMobileWorkItemDao;
	}

	/**
	 * Drops the mobile work item table.
	 */
	public void dropMobileWorkItemTable() {
		// TableUtils.dropTable(connectionSource, MobileWorkItem.class, true);
	}

	/**
	 * Closes the database connections and clears any cached DAOs.
	 */
	@Override
	public void close() {
		super.close();
		mMobileWorkItemDao = null;
	}
}
