package de.msponer.android.yac.marshaller;

import org.jdom.Element;
import org.yawlfoundation.yawl.engine.interfce.WorkItemRecord;
import org.yawlfoundation.yawl.util.JDOMUtil;

/**
 * Class is based on org.yawlfoundation.yawl.engine.interfce.Marshaller with
 * minor modifications due to mobile usage.
 * 
 * @author msponer
 */
public class MobileMarshaller {

	/**
	 * Instantiates a new mobile marshaller.
	 */
	protected MobileMarshaller() {
	}
	
	/**
	 * Unmarshal a work item.
	 *
	 * @param workItemXML the work item xml
	 * @return the work item record
	 */
	public static WorkItemRecord unmarshalWorkItem(String workItemXML) {
		return unmarshalWorkItem(JDOMUtil.stringToElement(workItemXML));
	}

	/**
	 * Unmarshal work item.
	 *
	 * @param workItemElement the work item element
	 * @return the work item record
	 */
	public static WorkItemRecord unmarshalWorkItem(Element workItemElement) {
		if (workItemElement == null) {
			return null;
		}	

		WorkItemRecord wir;
		String status = workItemElement.getChildText("status");
		String caseID = workItemElement.getChildText("caseid");
		String taskID = workItemElement.getChildText("taskid");
		String specURI = workItemElement.getChildText("specuri");
		String enablementTime = workItemElement.getChildText("enablementTime");
		if (caseID != null && taskID != null && specURI != null && enablementTime != null
				&& status != null) {

			wir = new WorkItemRecord(caseID, taskID, specURI, enablementTime, status);

			// wir.setExtendedAttributes(unmarshalWorkItemAttributes(workItemElement));
			wir.setUniqueID(workItemElement.getChildText("uniqueid"));
			wir.setTaskName(workItemElement.getChildText("taskname"));
			wir.setAllowsDynamicCreation(workItemElement.getChildText("allowsdynamiccreation"));
			wir.setRequiresManualResourcing(workItemElement
					.getChildText("requiresmanualresourcing"));
			wir.setCodelet(workItemElement.getChildText("codelet"));
			wir.setDeferredChoiceGroupID(workItemElement.getChildText("deferredChoiceGroupid"));
			wir.setSpecVersion(workItemElement.getChildText("specversion"));
			wir.setFiringTime(workItemElement.getChildText("firingTime"));
			wir.setStartTime(workItemElement.getChildText("startTime"));
			wir.setCompletionTimeMs(workItemElement.getChildText("completionTime"));
			wir.setEnablementTimeMs(workItemElement.getChildText("enablementTimeMs"));
			wir.setFiringTimeMs(workItemElement.getChildText("firingTimeMs"));
			wir.setStartTimeMs(workItemElement.getChildText("startTimeMs"));
			wir.setCompletionTimeMs(workItemElement.getChildText("completionTimeMs"));
			wir.setTimerTrigger(workItemElement.getChildText("timertrigger"));
			wir.setTimerExpiry(workItemElement.getChildText("timerexpiry"));
			wir.setStartedBy(workItemElement.getChildText("startedBy"));
			wir.setTag(workItemElement.getChildText("tag"));
			wir.setCustomFormURL(workItemElement.getChildText("customform"));

			String specid = workItemElement.getChildText("specidentifier");
			if (specid != null) {
				wir.setSpecIdentifier(specid);
			}	

			String resStatus = workItemElement.getChildText("resourceStatus");
			if (resStatus != null) {
				wir.setResourceStatus(resStatus);
			}	

			String edited = workItemElement.getChildText("edited");
			if (edited != null) {
				wir.setEdited(edited.equalsIgnoreCase("true"));
			}	

			Element data = workItemElement.getChild("data");
			if ((null != data) && (data.getContentSize() > 0)) {
				wir.setDataList((Element) data.getContent(0));
			}
			
			Element updateddata = workItemElement.getChild("updateddata");
			if ((null != updateddata) && (updateddata.getContentSize() > 0)) {
				wir.setUpdatedData((Element) updateddata.getContent(0));
			}
			
			Element logPredicate = workItemElement.getChild("logPredicate");
			if (logPredicate != null) {
				wir.setLogPredicateStarted(logPredicate.getChildText("start"));
				wir.setLogPredicateCompletion(logPredicate.getChildText("completion"));
			}

			return wir;
		}
		throw new IllegalArgumentException("Input element could not be parsed.");
	}
}
