package de.msponer.android.yac.marshaller;

import java.io.IOException;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.jdom.Element;
import org.yawlfoundation.yawl.engine.interfce.WorkItemRecord;
import org.yawlfoundation.yawl.resourcing.resource.Participant;
import org.yawlfoundation.yawl.util.JDOMUtil;

/**
 * Class is based on
 * org.yawlfoundation.yawl.resourcing.rsInterface.ResourceMarshaller with minor
 * modifications due to mobile usage.
 * 
 * @author msponer
 */
public class MobileResourceMarshaller {

	/**
	 * Instantiates a new mobile resource marshaller.
	 */
	public MobileResourceMarshaller() {
	}

	/**
	 * Converts the string passed to a JDOM Element and returns its child
	 * Elements.
	 *
	 * @param s the xml string to be converted
	 * @return a list of child elements of the converted element passed
	 */
	@SuppressWarnings("rawtypes")
	private List getChildren(String s) {
		if ((s == null) || (!s.trim().startsWith("<"))) {
			return null;
		}
		
		return JDOMUtil.stringToElement(s).getChildren();
	}

	/**
	 * Marshall participants.
	 *
	 * @param set the set
	 * @return the string
	 */
	public String marshallParticipants(Set<Participant> set) {
		StringBuilder xml = new StringBuilder("<participants>");
		if (set != null) {
			for (Participant p : set) {
				xml.append(p.toXML());
			}
		}	
		xml.append("</participants>");
		return xml.toString();
	}

	/**
	 * Unmarshall participants.
	 *
	 * @param xml the xml
	 * @return the sets the
	 */
	@SuppressWarnings("rawtypes")
	public Set<Participant> unmarshallParticipants(String xml) {
		Set<Participant> result = new HashSet<Participant>();

		// each child is one Participant (as xml)
		List eList = getChildren(xml);
		if (eList != null) {
			for (Object o : eList) {
				Participant p = new Participant();

				// repopulate the members from its xml
				p.reconstitute((Element) o);
				result.add(p);
			}
		}
		if (result.isEmpty()) {
			return null;
		}	
		return result;
	}

	/**
	 * Marshall work item records.
	 *
	 * @param set the set
	 * @return the string
	 */
	public String marshallWorkItemRecords(Set<WorkItemRecord> set) {
		StringBuilder xml = new StringBuilder("<workitemrecords>");
		if (set != null) {
			for (WorkItemRecord wir : set) {
				xml.append(wir.toXML());
			}
		}	
		xml.append("</workitemrecords>");
		return xml.toString();
	}

	/**
	 * Marshall work item records.
	 *
	 * @param list the list
	 * @return the string
	 */
	public String marshallWorkItemRecords(List<WorkItemRecord> list) {
		return marshallWorkItemRecords(new HashSet<WorkItemRecord>(list));
	}

	/**
	 * Unmarshall work item records.
	 *
	 * @param xml the xml
	 * @return the sets the
	 */
	public Set<WorkItemRecord> unmarshallWorkItemRecords(String xml) {
		Set<WorkItemRecord> result = new HashSet<WorkItemRecord>();

		// each child is one WorkItemRecord (as xml)
		@SuppressWarnings("rawtypes")
		List eList = getChildren(xml);
		if (eList != null) {
			for (Object o : eList) {
				Element e = (Element) o;
				result.add(MobileMarshaller.unmarshalWorkItem(e));
			}
		}
		if (result.isEmpty()) {
			return null;
		}	
		return result;
	}

	/**
	 * Unmarshall work item record.
	 *
	 * @param xml the xml
	 * @return the work item record
	 * @throws IOException Signals that an I/O exception has occurred.
	 */
	public WorkItemRecord unmarshallWorkItemRecord(String xml) throws IOException {
		return MobileMarshaller.unmarshalWorkItem(xml);
	}

}
