/**
 * 
 */
package de.msponer.android.yac.services.impl;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.sql.SQLException;
import java.util.Set;
import java.util.StringTokenizer;

import org.yawlfoundation.yawl.engine.interfce.WorkItemRecord;
import org.yawlfoundation.yawl.resourcing.resource.Participant;
import org.yawlfoundation.yawl.resourcing.rsInterface.ResourceGatewayClient;
import org.yawlfoundation.yawl.resourcing.rsInterface.WorkQueueGatewayClient;
import org.yawlfoundation.yawl.util.JDOMUtil;

import android.R;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Binder;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.util.Log;
import android.widget.Toast;

import com.j256.ormlite.android.apptools.OrmLiteBaseService;
import com.j256.ormlite.dao.Dao;
import com.j256.ormlite.table.TableUtils;

import de.msponer.android.yac.db.DatabaseHelper;
import de.msponer.android.yac.marshaller.MobileResourceMarshaller;
import de.msponer.android.yac.model.MobileWorkItem;
import de.msponer.android.yac.model.State;

/**
 * Android service encapsulating communication processes between YAC and YAWL.
 * Two sync modes are implemented (complete and live).
 * 
 * @author msponer
 * 
 */
public class YConnectionService extends OrmLiteBaseService<DatabaseHelper> {

	/**
	 * Sync-Action: Accept work item.
	 */
	public static final String SYNC_ACTION_ACCEPT_OFFER = "OfferAccepted";

	/**
	 * Sync-Action: Deallocate work item.
	 */
	public static final String SYNC_ACTION_DEALLOCATE = "Deallocated";

	/**
	 * Sync-Action: Skip work item.
	 */
	public static final String SYNC_ACTION_SKIP = "Skipped";

	/**
	 * Sync-Action: Pile work item.
	 */
	public static final String SYNC_ACTION_PILE = "Piled";

	/**
	 * Sync-Action: Start work item.
	 */
	public static final String SYNC_ACTION_START = "Started";

	/**
	 * Sync-Action: Save work item.
	 */
	public static final String SYNC_ACTION_SAVE = "Saved";

	/**
	 * Sync-Action: Complete work item.
	 */
	public static final String SYNC_ACTION_COMPLETE = "Completed";

	/**
	 * Sync-Action: Suspend work item.
	 */
	public static final String SYNC_ACTION_SUSPEND = "Suspended";

	/**
	 * Sync-Action: Unsuspend work item.
	 */
	public static final String SYNC_ACTION_UNSUSPEND = "Unsuspended";

	/**
	 * The application tag.
	 */
	private static final String TAG = "YConnectionService";

	/**
	 * Unique Identification Number for the Notification. We use it on
	 * Notification start, and to cancel it.
	 */
	private static final String NOTIFICATION = "YConnectionService Notification";

	/**
	 * The service connection binder.
	 */
	private final IBinder mYConnectionServiceBinder = new YConnectionServiceBinder();

	/**
	 * The callback handler.
	 */
	private Handler mCallbackHandler;

	/**
	 * The resource gateway interface.
	 */
	private ResourceGatewayClient mResourceGateway;

	/**
	 * The work queue gateway interface.
	 */
	private WorkQueueGatewayClient mWorkQueueGateway;

	/**
	 * A marshaller for mobile work items.
	 */
	private MobileResourceMarshaller mMobResMarshaller;

	/**
	 * The YAWL servcer base URL.
	 */
	private String mServerBaseURL;

	/**
	 * The complete resource gateway url.
	 */
	private String mResourceGatewayUrl;

	/**
	 * The complete work queue gateway url.
	 */
	private String mWorkQueueGatewayUrl;

	// private String interfaceBGatewayUrl;

	/**
	 * The current resource gateway handle.
	 */
	private String mCurrentRGHandle;

	/**
	 * The current work queue gateway handle.
	 */
	private String mCurrentWQGHandle;

	// private String currentIBGHandle;

	/**
	 * A resource gateway connection flag.
	 */
	private boolean mRGConnected;

	/**
	 * A work queue gateway connection flag.
	 */
	private boolean mWQGConnected;

	// private boolean ibgConnected;

	/**
	 * The client app account name.
	 */
	private String mClientAppAccount;

	/**
	 * The client app password.
	 */
	private String mClientAppPassword;

	/**
	 * The participant's user name.
	 */
	private String mUsername;

	/**
	 * The participant's password.
	 */
	private String mPassword;

	/**
	 * Android notification manager.
	 */
	private NotificationManager mNM;

	/**
	 * The mobile work item dao.
	 */
	private Dao<MobileWorkItem, Integer> mMobileWorkItemDao;

	/**
	 * The current YAWL participant.
	 */
	private Participant mParticipant;

	/**
	 * The connection binder class.
	 * 
	 * @author msponer
	 * 
	 */
	public class YConnectionServiceBinder extends Binder {
		/**
		 * Returns the service connection.
		 * 
		 * @return The current service connection.
		 */
		public YConnectionService getYConnectionService() {
			return YConnectionService.this;
		}

		/**
		 * Sets the activity callback handler.
		 * 
		 * @param callback
		 *            The callback handler.
		 */
		public void setActivityCallbackHandler(final Handler callback) {
			mCallbackHandler = callback;
		}
	}

	@Override
	public void onCreate() {
		// mNM = (NotificationManager)getSystemService(NOTIFICATION_SERVICE);

		// Display a notification about us starting. We put an icon in the
		// status bar.
		// showNotification();

		SharedPreferences sharedPreferences = getSharedPreferences(
				"de.msponer.android.yac_preferences", MODE_PRIVATE);

		mServerBaseURL = sharedPreferences
				.getString("prefs_server_url", "http://marcels-mbp:8080/");
		Log.d(TAG, mServerBaseURL);

		String clientAppAccount = sharedPreferences.getString("prefs_client_app_account", "YAC");
		Log.d(TAG, clientAppAccount);

		String clientAppPassword = sharedPreferences.getString("prefs_client_app_password",
				"YACpassword");
		Log.d(TAG, clientAppPassword);

		String username = sharedPreferences.getString("prefs_username", "mspone2s");
		Log.d(TAG, username);

		String password = sharedPreferences.getString("prefs_password", "mspone2s");
		Log.d(TAG, password);

		this.mResourceGatewayUrl = mServerBaseURL + "resourceService/gateway";
		this.mWorkQueueGatewayUrl = mServerBaseURL + "resourceService/workqueuegateway";
		// this.interfaceBGatewayUrl = serverBaseURL + "yawl/ib";

		this.mUsername = username;
		this.mPassword = password;

		this.mClientAppAccount = clientAppAccount;
		this.mClientAppPassword = clientAppPassword;

		// this.resourceGateway = new
		// ResourceGatewayClientAdapter(this.resourceGatewayUrl);
		// this.workQueueGateway = new
		// WorkQueueGatewayClientAdapter(this.workQueueGatewayUrl);
		// this.interfaceBGateway = new
		// InterfaceB_EnvironmentBasedClient(this.interfaceBGatewayUrl);

		this.mResourceGateway = new ResourceGatewayClient(this.mResourceGatewayUrl);
		this.mWorkQueueGateway = new WorkQueueGatewayClient(this.mWorkQueueGatewayUrl);

		mMobResMarshaller = new MobileResourceMarshaller();

		try {
			this.mMobileWorkItemDao = getHelper().getMobileWorkItemDao();
		} catch (SQLException e) {
			Log.e(TAG, "Couldn't create Workitem DAO", e.fillInStackTrace());
		}

	}

	@Override
	public void onDestroy() {
		super.onDestroy();
	}

	@Override
	public IBinder onBind(Intent intent) {
		return mYConnectionServiceBinder;
	}

	/**
	 * Initializes the connections to the resource and work queue gateway and
	 * passes participant's user credentials.
	 */
	public void yConnect() {

		try {
			Log.d(TAG, "Connecting to ResourceGateway: " + mResourceGatewayUrl);
			mCurrentRGHandle = mResourceGateway.connect(mClientAppAccount, mClientAppPassword);
			mRGConnected = mResourceGateway.successful(mCurrentRGHandle);
			Log.d(TAG, "Current rgHandle: " + mCurrentRGHandle);

			Log.d(TAG, "Connecting to WorkQueueGateway: " + mWorkQueueGatewayUrl);
			mCurrentWQGHandle = mWorkQueueGateway.connect(mClientAppAccount, mClientAppPassword);
			mWQGConnected = mWorkQueueGateway.successful(mCurrentWQGHandle);
			Log.d(TAG, "Current wqgHandle: " + mCurrentWQGHandle);

			// Log.d(TAG, "Connecting to InterfaceBGateway: " +
			// this.interfaceBGatewayUrl);
			// this.currentIBGHandle =
			// this.interfaceBGateway.connect(clientAppAccount,
			// clientAppPassword);
			// this.ibgConnected =
			// this.interfaceBGateway.successful(this.currentIBGHandle);
			// Log.d(TAG, "Current ibgHandle: " + this.currentIBGHandle);

			if (mParticipant == null) {
				String participantString = mResourceGateway.getParticipantFromUserID(mUsername,
						mCurrentRGHandle);
				Log.d(TAG, participantString);
				if (participantString != null && participantString.length() > 0) {
					mParticipant = new Participant(JDOMUtil.stringToElement(participantString));
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Disconnects the YAWL gateways.
	 */
	public void yDisconnect() {
		try {
			this.mResourceGateway.disconnect(this.mCurrentRGHandle);
			this.mWorkQueueGateway.disconnect(this.mCurrentWQGHandle);

			this.mCurrentRGHandle = "";
			this.mCurrentWQGHandle = "";
			// this.currentIBGHandle = "";

			this.mRGConnected = false;
			this.mWQGConnected = false;
			// this.ibgConnected = false;

			setParticipant(null);

			Log.d(TAG, "YConnectionManger disconnected.");
		} catch (Exception e) {
			Log.e(TAG, "Error during yDissconnect()");
		}
	}

	/**
	 * Shows notification using android notification manager.
	 */
	@SuppressWarnings("unused")
	private void showNotification() {

		// Set the icon, scrolling text and timestamp
		Notification notification = new Notification(R.drawable.stat_notify_sync, NOTIFICATION,
				System.currentTimeMillis());

		// The PendingIntent to launch our activity if the user selects this
		// notification
		PendingIntent contentIntent = PendingIntent.getActivity(this, 0, null, 0);

		// Set the info for the views that show in the notification panel.
		notification.setLatestEventInfo(this, TAG, "started...", contentIntent);

		// Send the notification.
		mNM.notify(77, notification);

	}

	/**
	 * Performs a complete sync between YAC and YAWL. 3 phases are proceeded: <br>
	 * 1. All modified work items are submitted. <br>
	 * 2. The DB is reseted. <br>
	 * 3. All work items are fetched.
	 */
	public void doSync() {

		new Thread() {
			@Override
			public void run() {

				Message msg = new Message();
				Bundle bundle = new Bundle();

				if (areAllInterfacesConnected()) {

					try {
						submitAllModifiedWorkitems();

						fetchAllWorkitems();

						bundle.putBoolean("SYNC_RESULT", true);
						msg.setData(bundle);
						mCallbackHandler.sendMessage(msg);

					} catch (Exception e) {
						bundle.putBoolean("SYNC_RESULT", false);

						StringWriter sw = new StringWriter();
						e.printStackTrace(new PrintWriter(sw));
						String exceptionAsStrting = sw.toString();

						// bundle.putString("SYNC_OUTPUT",
						// e.fillInStackTrace().toString());
						bundle.putString("SYNC_OUTPUT", exceptionAsStrting);
						msg.setData(bundle);
						mCallbackHandler.sendMessage(msg);
						Log.e(TAG, e.getMessage(), e.fillInStackTrace());
					}
				} else {
					Log.e(TAG,
							"Connection to the required YAWL-Services could not be established...");
					bundle.putBoolean("SYNC_RESULT", false);
					bundle.putString("SYNC_OUTPUT",
							"Connection to the required YAWL-Services could not be established...");
					msg.setData(bundle);
					mCallbackHandler.sendMessage(msg);
				}
			}
		} .start();
	}

	/**
	 * Performs a live sync for a specified work item. After submitting the
	 * modifications the new status is retrieved and db is updated.
	 * 
	 * @param wi
	 *            The work item to sync.
	 * @return the work-item primary key.
	 */
	public int liveSyncWorkitem(final MobileWorkItem wi) {
		try {
			yConnect();

			if (areAllInterfacesConnected()) {
				String submittedId = submitModifiedWorkitem(wi);
				mMobileWorkItemDao.delete(wi);
				if (!wi.getResourceStatus().equals(MobileWorkItem.STATUS_COMPLETE)) {
					String workItemString = mWorkQueueGateway.getWorkItem(submittedId,
							mCurrentWQGHandle);
					WorkItemRecord unmarshalWorkItem = mMobResMarshaller
							.unmarshallWorkItemRecord(workItemString);
					if (unmarshalWorkItem != null) {
						return insertWorkitem(unmarshalWorkItem);
					} else {
						throw new Exception("Receiving and unmarshalling workitem incomplete!");
					}
				}
				Toast.makeText(getApplicationContext(), "Live-Sync successfully complete!",
						Toast.LENGTH_SHORT).show();
			} else {
				// throw new Exception();
				throw new Exception("Connection to YAWL-Engine could not be established...");
			}
		} catch (Exception e) {
			Toast.makeText(getApplicationContext(), "Live-Sync incomplete!", Toast.LENGTH_SHORT)
					.show();
			Log.e(TAG, wi.getID());
			Log.e(TAG, e.getMessage(), e.fillInStackTrace());
		}
		return -1;
	}

	/**
	 * Iterates over all modified work items and invokes the submit operation.
	 * 
	 * @throws Exception
	 *             Error during submission.
	 */
	private void submitAllModifiedWorkitems() throws Exception {

		Log.i(TAG, "Submitting ALL modified Workitems...");

		for (MobileWorkItem wi : mMobileWorkItemDao.queryForAll()) {
			if (wi.getState() == State.MODIFIED) {
				submitModifiedWorkitem(wi);
			}
		}
	}

	/**
	 * Submits all modified work items to YAWL using the work queue gateway
	 * interface.
	 * 
	 * @param wi
	 *            The work item to submit.
	 * @return The work queue result output.
	 * @throws Exception
	 *             Error during submission.
	 */
	private String submitModifiedWorkitem(MobileWorkItem wi) throws Exception {

		Log.d(TAG, "Syncing item: " + wi.getID());
		Log.d(TAG, "Sync-String: " + wi.getSyncQueue());

		StringTokenizer tokenizer = new StringTokenizer(wi.getSyncQueue(), ",", false);
		Log.d(TAG, "Tokenizer-Count:" + tokenizer.countTokens());
		while (tokenizer.hasMoreTokens()) {
			String action = (String) tokenizer.nextToken();
			Log.d(TAG, wi.getID() + " - Action: " + action);

			if (action != null && action.length() > 0) {
				String result = "";
				if (action.equals(SYNC_ACTION_ACCEPT_OFFER)) {
					Log.i(TAG, "Accepting offer for WorkItem " + wi.getID());
					result = mWorkQueueGateway.acceptOffer(getParticipant().getID(), wi.getID(),
							mCurrentWQGHandle);
					if (result.startsWith("<success/>")) {
						wi.removeSyncAction(action);
					}
				} else if (action.equals(SYNC_ACTION_DEALLOCATE)) {
					Log.i(TAG, "Deallocating WorkItem " + wi.getID());

					result = mWorkQueueGateway.deallocateItem(getParticipant().getID(), wi.getID(),
							mCurrentWQGHandle);

					if (result.contains("<success/>")) {
						wi.removeSyncAction(action);
					}
				} else if (action.equals(SYNC_ACTION_SKIP)) {
					Log.i(TAG, "Skipping WorkItem " + wi.getID());
					result = mWorkQueueGateway.skipItem(getParticipant().getID(), wi.getID(),
							mCurrentWQGHandle);

					if (result.contains("<success/>")) {
						wi.removeSyncAction(action);
					}
				} else if (action.equals(SYNC_ACTION_PILE)) {
					/*
					 * Interface-Method Bug!
					 */
					Log.i(TAG, "Piling WorkItem " + wi.getID());
					// result =
					// workQueueGateway.pileItem(getParticipant().getID(),
					// wi.getID(), currentWQGHandle);
					// Log.d(TAG, "TaskId: " + wi.getTaskID());
					// Log.d(TAG, result);
					//
					// if(result.contains("<success/>")){
					// wi.removeSyncAction(action);
					// }
				} else if (action.equals(SYNC_ACTION_START)) {
					Log.i(TAG, "Starting WorkItem " + wi.getID());

					String startedItemString = mWorkQueueGateway.startItem(
							getParticipant().getID(), wi.getID(), mCurrentWQGHandle);

					WorkItemRecord startedItem = mMobResMarshaller
							.unmarshallWorkItemRecord(startedItemString);

					wi.setCaseID(startedItem.getCaseID());
					wi.setTaskID(startedItem.getTaskID());
					mMobileWorkItemDao.update(wi);

					if (wi.getID().contains(".")) {
						wi.removeSyncAction(action);
					}
				} else if (action.equals(SYNC_ACTION_SAVE)) {
					Log.i(TAG, "Saving updated data for WorkItem " + wi.getID());
					result = mWorkQueueGateway.updateWorkItemData(wi.getID(),
							wi.getUpdatedDataString(), mCurrentWQGHandle);
				} else if (action.equals(SYNC_ACTION_COMPLETE)) {
					Log.i(TAG, "Completing WorkItem " + wi.getID());
					result = mWorkQueueGateway.completeItem(this.mParticipant.getID(), wi.getID(),
							mCurrentWQGHandle);
				} else if (action.equals(SYNC_ACTION_SUSPEND)) {
					Log.i(TAG, "Suspending WorkItem " + wi.getID());
					result = mWorkQueueGateway.suspendItem(this.mParticipant.getID(), wi.getID(),
							mCurrentWQGHandle);
				} else if (action.equals(SYNC_ACTION_UNSUSPEND)) {
					Log.i(TAG, "Completing WorkItem " + wi.getID());
					result = mWorkQueueGateway.unsuspendItem(this.mParticipant.getID(), wi.getID(),
							mCurrentWQGHandle);
				}

				Log.i(TAG, result);
			}
		}
		return wi.getID();
	}

	/**
	 * Resets the db and fetches all available work items for the current
	 * participant.
	 * 
	 * @throws Exception
	 *             Error during transmission.
	 */
	private void fetchAllWorkitems() throws Exception {

		Log.d(TAG, "Mobile-WorkItem-Count before sync: " + mMobileWorkItemDao.countOf());

		resetDb();

		Log.d(TAG, "Doing sync for participant-id: " + getParticipant().getID());

		for (int i = 0; i < 4; i++) {
			String queuedWorkItemsString = mWorkQueueGateway.getQueuedWorkItems(getParticipant()
					.getID(), i, mCurrentWQGHandle);
			Set<WorkItemRecord> queuedWorkItems = mMobResMarshaller
					.unmarshallWorkItemRecords(queuedWorkItemsString);

			if (queuedWorkItems != null) {
				Log.d(TAG, "Queue: " + i + ", Size: " + queuedWorkItems.size());

				for (WorkItemRecord wir : queuedWorkItems) {
					insertWorkitem(wir);
				}
			}
		}

		Log.d(TAG, "Mobile-WorkItem-Count after sync: " + mMobileWorkItemDao.countOf());
	}

	/**
	 * Inserts the given work item into the database.
	 * 
	 * @param wir
	 *            The work item to insert.
	 * @throws Exception
	 *             Error during insertion.
	 * @return The primary key of the inserted mobile workitem
	 */
	private int insertWorkitem(WorkItemRecord wir) throws Exception {

		MobileWorkItem mobWir = new MobileWorkItem(wir);

		Log.d(TAG, "Applying Dataschema-String");
		String workItemDataSchema = mWorkQueueGateway.getWorkItemDataSchema(wir.getID(),
				mCurrentWQGHandle);
		// Log.d(TAG, workItemDataSchema);
		mobWir.setDataSchemaString(workItemDataSchema);

		String workitemCaseData = mWorkQueueGateway.getCaseData(wir.getCaseID(), mCurrentWQGHandle);
		mobWir.setCaseDataString(workitemCaseData);

		mMobileWorkItemDao.create(mobWir);

		Log.d(TAG, "Created new DB-Entry for WorkItemRecord: " + mobWir.getID());

		return mobWir.getPrimaryKey();
	}

	/**
	 * Resets the DB, all mobile work items are dropped.
	 */
	public void resetDb() {
		try {
			Log.d(TAG, "Dropping MobileWorkItem Table ...");
			TableUtils.dropTable(getConnectionSource(), MobileWorkItem.class, true);

			Log.d(TAG, "Creating new MobileWorkItem Table ...");
			TableUtils.createTable(getConnectionSource(), MobileWorkItem.class);
		} catch (SQLException e) {
			Log.e(TAG, e.getMessage(), e.fillInStackTrace());
		}
	}

	/**
	 * Sets the resource gateway.
	 * 
	 * @param resourceGateway
	 *            The resource gateway to set.
	 */
	public void setResourceGateway(ResourceGatewayClient resourceGateway) {
		this.mResourceGateway = resourceGateway;
	}

	/**
	 * Returns the resource gateway.
	 * 
	 * @return the resource gateway.
	 */
	public ResourceGatewayClient getResourceGateway() {
		return mResourceGateway;
	}

	/**
	 * Sets the work queue gatway.
	 * 
	 * @param workQueueGateway
	 *            the work queue gatway.
	 */
	public void setWorkQueueGateway(WorkQueueGatewayClient workQueueGateway) {
		this.mWorkQueueGateway = workQueueGateway;
	}

	/**
	 * Returns the work queue gatway.
	 * 
	 * @return the work queue gatway.
	 */
	public WorkQueueGatewayClient getWorkQueueGateway() {
		return mWorkQueueGateway;
	}

	/**
	 * Returns the participant's username.
	 * 
	 * @return the participant's username
	 */
	public String getUsername() {
		return mUsername;
	}

	/**
	 * Sets the participant's username.
	 * 
	 * @param username
	 *            the participant's username
	 */
	public void setUsername(String username) {
		this.mUsername = username;
	}

	/**
	 * Returns the participant's password.
	 * 
	 * @return the participant's password
	 */
	public String getPassword() {
		return mPassword;
	}

	/**
	 * Sets the participant's password.
	 * 
	 * @param password
	 *            the participant's password
	 */
	public void setPassword(String password) {
		this.mPassword = password;
	}

	/**
	 * Sets the resource gateway url.
	 * 
	 * @param resourceGatewayUrl
	 *            the resource gateway url.
	 */
	public void setResourceGatewayUrl(String resourceGatewayUrl) {
		this.mResourceGatewayUrl = resourceGatewayUrl;
	}

	/**
	 * Returns the resource gateway url.
	 * 
	 * @return the resource gateway url.
	 */
	public String getResourceGatewayUrl() {
		return mResourceGatewayUrl;
	}

	/**
	 * Sets the work queue gateway url.
	 * 
	 * @param workQueueGatewayUrl
	 *            the work queue gateway url.
	 */
	public void setWorkQueueGatewayUrl(String workQueueGatewayUrl) {
		this.mWorkQueueGatewayUrl = workQueueGatewayUrl;
	}

	/**
	 * Returns the work queue gateway url.
	 * 
	 * @return the work queue gateway url.
	 */
	public String getWorkQueueGatewayUrl() {
		return mWorkQueueGatewayUrl;
	}

	/**
	 * true if resource gateway is connected.
	 * 
	 * @return true if resource gateway is connected.
	 */
	public boolean isRgConnected() {
		return mRGConnected;
	}

	/**
	 * Sets the resource gateway connection flag.
	 * 
	 * @param rgConnected
	 *            the resource gateway connection flag
	 */
	public void setRgConnected(boolean rgConnected) {
		this.mRGConnected = rgConnected;
	}

	/**
	 * true if work queue gateway is connected.
	 * 
	 * @return true if work queue gateway is connected.
	 */
	public boolean isWqgConnected() {
		return mWQGConnected;
	}

	/**
	 * Sets the work queue gateway connection flag.
	 * 
	 * @param wqgConnected
	 *            the work queue gateway connection flag
	 */
	public void setWqgConnected(boolean wqgConnected) {
		this.mWQGConnected = wqgConnected;
	}

	/**
	 * Returns the current resource gateway handle.
	 * 
	 * @return the current resource gateway handle
	 */
	public String getCurrentRGHandle() {
		return mCurrentRGHandle;
	}

	/**
	 * Returns the current work queue gateway handle.
	 * 
	 * @return the current work queue gateway handle
	 */
	public String getCurrentWQGHandle() {
		return mCurrentWQGHandle;
	}

	// public void setInterfaceBGateway(InterfaceB_EnvironmentBasedClient
	// interfaceBGateway) {
	// this.interfaceBGateway = interfaceBGateway;
	// }
	//
	// public InterfaceB_EnvironmentBasedClient getInterfaceBGateway() {
	// return interfaceBGateway;
	// }

	// public void setInterfaceBGatewayUrl(String interfaceBGatewayUrl) {
	// this.interfaceBGatewayUrl = interfaceBGatewayUrl;
	// }
	//
	// public String getInterfaceBGatewayUrl() {
	// return interfaceBGatewayUrl;
	// }
	//
	// public void setCurrentIBGHandle(String currentIBGHandle) {
	// this.currentIBGHandle = currentIBGHandle;
	// }
	//
	// public String getCurrentIBGHandle() {
	// return currentIBGHandle;
	// }
	//
	// public void setIbgConnected(boolean ibgConnected) {
	// this.ibgConnected = ibgConnected;
	// }
	//
	// public boolean isIbgConnected() {
	// return ibgConnected;
	// }

	/**
	 * Returns true if all interfaces are connected.
	 * 
	 * @return true if all interfaces are connected.
	 */
	public boolean areAllInterfacesConnected() {
		return
		// ibgConnected &&
		mRGConnected && mWQGConnected;
	}

	/**
	 * Sets the current participant.
	 * 
	 * @param participant
	 *            the current participant
	 */
	public void setParticipant(Participant participant) {
		this.mParticipant = participant;
	}

	/**
	 * Returns the current participant.
	 * 
	 * @return the current participant
	 */
	public Participant getParticipant() {
		return mParticipant;
	}
}
