/**
 * 
 */
package de.msponer.android.yac.xml;

import java.util.List;

import org.jdom.Element;

import android.util.Log;

/**
 * Serializer for work item element entries. Takes the element entry root,
 * visits all child elements and generates XML data.
 * 
 * @author msponer
 * 
 */
public class WorkItemElementEntrySerializer {

	/**
	 * Hidden constructor.
	 */
	protected WorkItemElementEntrySerializer() {
	}
	
	/**
	 * The class tag.
	 */
	private static final String TAG = "WorkItemElementEntrySerializer";

	/**
	 * Starts the serialize process.
	 * 
	 * @param workItemElementEntryRoot
	 *            The work item element entry root.
	 * @return The resulting XML data element.
	 */
	public static Element serialize(WorkItemElementEntry workItemElementEntryRoot) {

		Element root = new Element(workItemElementEntryRoot.getName());
		Log.d(TAG, "Created-Root: " + workItemElementEntryRoot.getName());

		traversal(root, workItemElementEntryRoot.getSubElementEntryList());

		return root;
	}

	/**
	 * Visits the given list of entries and adds the content to the XML data
	 * element. If child elements are present, the method will be invoked
	 * recursively with the sub-entry list.
	 * 
	 * @param father
	 *            The XML father element.
	 * @param childList
	 *            The entry list.
	 */
	public static void traversal(Element father, List<WorkItemElementEntry> childList) {

		for (WorkItemElementEntry entry : childList) {
			Element child = new Element(entry.getName());
			Log.d(TAG, "Created: " + entry.getName());

			if (entry.getValue() != null && entry.getValue().length() > 0) {
				Log.d(TAG, "Childvalue: " + entry.getValue());
				child.setText(entry.getValue());
			}

			father.addContent(child);

			if (entry.getSubElementEntryList() != null) {
				traversal(child, entry.getSubElementEntryList());
			}
		}
	}
}
